@@ -2,7 +2,7 @@
  * Configurable ps-like program.
  * Conditional display routines.
  *
- * Copyright (c) 2008 David I. Bell
+ * Copyright (c) 2010 David I. Bell
  * Permission is granted to use, distribute, or modify this source,
  * provided that this copyright notice remains intact.
  */
@@ -34,10 +34,10 @@
 /*
  * Static variables.
  */
-static	BOOL	cond_flag;		/* TRUE if have condition tree */
-static	TREE	cond_tree;		/* condition tree */
-static	NODE	badnode;		/* special node for errors */
-static	VALUE	badvalue;		/* special value for errors */
+static	BOOL	condFlag;		/* TRUE if have condition tree */
+static	TREE	condTree;		/* condition tree */
+static	NODE	badNode;		/* special node for errors */
+static	VALUE	badValue;		/* special value for errors */
 
 
 /*
@@ -63,8 +63,8 @@ static	TOKEN	ParseNewToken(TREE * tree);
 static	TOKEN	ParseToken(TREE * tree);
 static	TOKEN	ParseNumber(TREE * tree);
 static	TOKEN	BadToken(TREE * tree, char * msg);
-static	VALUE	EvaluateComparison(TREE * tree, NODE * node);
-static	VALUE	EvaluateFunction(TREE * tree, NODE * node);
+static	VALUE	EvaluateComparison(TREE * tree, const NODE * node);
+static	VALUE	EvaluateFunction(TREE * tree, const NODE * node);
 static	BOOL	ConvertType(VALUE val, int type, VALUE * retval);
 
 static	VALUE	FunctionMatch(TREE * tree, VALUE arg1, VALUE arg2);
@@ -74,7 +74,7 @@ static	VALUE	FunctionStrlen(TREE * tree, VALUE arg);
 static	VALUE	FunctionAbs(TREE * tree, VALUE arg);
 static	VALUE	FunctionStr(TREE * tree, VALUE arg);
 static	VALUE	FunctionCmp(TREE * tree, VALUE arg1, VALUE arg2);
-static	VALUE	FunctionMy(TREE * tree, NODE *node);
+static	VALUE	FunctionMy(TREE * tree, const NODE *node);
 
 
 /*
@@ -89,76 +89,98 @@ IsShownProcess(const PROC * proc)
 	int	i;
 	VALUE	value;
 
-	if (my_procs && (proc->uid != my_uid))
+	/*
+	 * If this is a thread and threads are not being shown then
+	 * we don't want it.
+	 */
+	if (proc->isThread && !showThreads)
 		return FALSE;
 
-	if (no_self && (proc->pid == my_pid))
+	/*
+	 * If we are showing threads and this is a main process and
+	 * there is more than 1 thread then we don't want it.
+	 */
+	if (showThreads && !proc->isThread && (proc->threadCount > 1))
 		return FALSE;
 
-	if (pid_count)
-	{
-		for (i = 0; i < pid_count; i++)
-		{
-			if (pid_list[i] == proc->pid)
-				break;
-		}
-
-		if (i == pid_count)
-			return FALSE;
-	}
-
-	if (user_count)
-	{
-		for (i = 0; i < user_count; i++)
-		{
-			if (user_list[i] == proc->uid)
-				break;
-		}
-
-		if (i == user_count)
-			return FALSE;
-	}
-
-	if (group_count)
-	{
-		for (i = 0; i < group_count; i++)
-		{
-			if (group_list[i] == proc->gid)
-				break;
-		}
-
-		if (i == group_count)
-			return FALSE;
-	}
-
-	if (program_count)
-	{
-		for (i = 0; i < program_count; i++)
-		{
-			if (strcmp(program_list[i], proc->program) == 0)
-				break;
-		}
-
-		if (i == program_count)
-			return FALSE;
-	}
-
-	if (active_only && !proc->isactive)
+	/*
+	 * If we are showing threads and this is a thread but there is
+	 * only one thread then we don't want it (since we will show
+	 * the main process in that case).
+	 */
+	if (showThreads && proc->isThread && (proc->threadCount <= 1))
 		return FALSE;
 
-	if (!cond_flag)
+	if (myProcs && (proc->uid != myUid))
+		return FALSE;
+
+	if (noSelf && (proc->pid == myPid))
+		return FALSE;
+
+	if (pidCount)
+	{
+		for (i = 0; i < pidCount; i++)
+		{
+			if (pidList[i] == proc->pid)
+				break;
+		}
+
+		if (i == pidCount)
+			return FALSE;
+	}
+
+	if (userCount)
+	{
+		for (i = 0; i < userCount; i++)
+		{
+			if (userList[i] == proc->uid)
+				break;
+		}
+
+		if (i == userCount)
+			return FALSE;
+	}
+
+	if (groupCount)
+	{
+		for (i = 0; i < groupCount; i++)
+		{
+			if (groupList[i] == proc->gid)
+				break;
+		}
+
+		if (i == groupCount)
+			return FALSE;
+	}
+
+	if (programCount)
+	{
+		for (i = 0; i < programCount; i++)
+		{
+			if (strcmp(programList[i], proc->program) == 0)
+				break;
+		}
+
+		if (i == programCount)
+			return FALSE;
+	}
+
+	if (activeOnly && !GetIsActive(proc))
+		return FALSE;
+
+	if (!condFlag)
 		return TRUE;
 
-	tree = &cond_tree;
+	tree = &condTree;
 	tree->proc = proc;
 
 	value = EvaluateNode(tree, tree->root);
 
 	if ((value.type == VALUE_NUMBER) || (value.type == VALUE_BOOLEAN))
-		return (value.intval != 0);
+		return (value.intVal != 0);
 
 	if (value.type == VALUE_STRING)
-		return (*value.strval != '\0');
+		return (*value.strVal != '\0');
 
 	return FALSE;
 }
@@ -172,10 +194,10 @@ IsShownProcess(const PROC * proc)
 USEFLAG
 GetConditionUseFlags(void)
 {
-	if (!cond_flag)
+	if (!condFlag)
 		return USE_NONE;
 
-	return GetNodeUseFlags(cond_tree.root);
+	return GetNodeUseFlags(condTree.root);
 }
 
 
@@ -199,11 +221,11 @@ GetNodeUseFlags(const NODE * node)
 		case OP_COLUMN_BASE:
 		case OP_COLUMN_SHOW:
 		case OP_COLUMN_TEST:
-			flags |= node->column->useflag;
+			flags |= node->column->useFlag;
 			break;
 
 		case OP_FUNCTION:
-			if (node->intval == FUNC_MY)
+			if (node->intVal == FUNC_MY)
 				flags |= USE_SELF;
 
 		default:
@@ -231,43 +253,43 @@ GetNodeUseFlags(const NODE * node)
  * concerning the specified process starting with the specified node.
  */
 VALUE
-EvaluateNode(TREE * tree, NODE * node)
+EvaluateNode(TREE * tree, const NODE * node)
 {
 	NODE *		left;
 	NODE *		right;
-	int		lefttype;
-	int		righttype;
-	int		valtype;
+	int		leftType;
+	int		rightType;
+	int		valType;
 	VALUE		value;
-	VALUE		leftval;
-	VALUE		rightval;
+	VALUE		leftVal;
+	VALUE		rightVal;
 	const char *	cp;
 
 	if (node == NULL)
-		return badvalue;
+		return badValue;
 
 	left = node->child[0];
 	right = node->child[1];
 
-	lefttype = VALUE_NONE;
-	righttype = VALUE_NONE;
-	valtype = VALUE_NONE;
+	leftType = VALUE_NONE;
+	rightType = VALUE_NONE;
+	valType = VALUE_NONE;
 
 	/*
-	 * First specify the required types of subtree expressions
+	 * First specify the required types of subTree expressions
 	 * and whether we need to evaluate them.
 	 */
 	switch (node->op)
 	{
 		case OP_NOT:
-			lefttype = VALUE_BOOLEAN;
-			valtype = VALUE_BOOLEAN;
+			leftType = VALUE_BOOLEAN;
+			valType = VALUE_BOOLEAN;
 			break;
 
 		case OP_NEGATE:
 		case OP_COMPLEMENT:
-			lefttype = VALUE_NUMBER;
-			valtype = VALUE_NUMBER;
+			leftType = VALUE_NUMBER;
+			valType = VALUE_NUMBER;
 			break;
 
 		case OP_ADD:
@@ -277,47 +299,47 @@ EvaluateNode(TREE * tree, NODE * node)
 		case OP_MODULO:
 		case OP_AND:
 		case OP_XOR:
-			lefttype = VALUE_NUMBER;
-			righttype = VALUE_NUMBER;
-			valtype = VALUE_NUMBER;
+			leftType = VALUE_NUMBER;
+			rightType = VALUE_NUMBER;
+			valType = VALUE_NUMBER;
 			break;
 
 		case OP_OROR:
 		case OP_ANDAND:
-			lefttype = VALUE_BOOLEAN;
-			righttype = VALUE_BOOLEAN;
-			valtype = VALUE_BOOLEAN;
+			leftType = VALUE_BOOLEAN;
+			rightType = VALUE_BOOLEAN;
+			valType = VALUE_BOOLEAN;
 			break;
 
 		case OP_ALTERNATION:
-			lefttype = VALUE_BOOLEAN;
+			leftType = VALUE_BOOLEAN;
 			break;
 
 		case OP_STRING:
 			value.type = VALUE_STRING;
-			value.strval = node->strval;
+			value.strVal = node->strVal;
 
 			return value;
 
 		case OP_NUMBER:
 			value.type = VALUE_NUMBER;
-			value.intval = node->intval;
+			value.intVal = node->intVal;
 
 			return value;
 
 		case OP_COLUMN_TEST:
 			value.type = VALUE_BOOLEAN;
-			value.intval =  node->column->testfunc(tree->proc);
+			value.intVal =  node->column->testFunc(tree->proc);
 
 			return value;
 
 		case OP_COLUMN_BASE:
-			node->column->evalfunc(tree->proc, &value);
+			node->column->evalFunc(tree->proc, &value);
 
 			return value;
 
 		case OP_COLUMN_SHOW:
-			cp = node->column->showfunc(tree->proc);
+			cp = node->column->showFunc(tree->proc);
 
 			/*
 			 * This is a bit of a hack.
@@ -340,7 +362,7 @@ EvaluateNode(TREE * tree, NODE * node)
 			 * we end the current display of processes.
 			 */
 			value.type = VALUE_STRING;
-			value.strval = cp;
+			value.strVal = cp;
 
 			return value;
 
@@ -356,30 +378,43 @@ EvaluateNode(TREE * tree, NODE * node)
 			return EvaluateFunction(tree, node);
 
 		default:
-			return badvalue;
+			return badValue;
 	}
 
+	/*
+	 * Initialize the values to make the compiler happy.
+	 */
+	leftVal.type = VALUE_NONE;
+	leftVal.intVal = 0;
+	leftVal.strVal = 0;
+	leftVal.column = 0;
+
+	rightVal.type = VALUE_NONE;
+	rightVal.intVal = 0;
+	rightVal.strVal = 0;
+	rightVal.column = 0;
+
 	/*
 	 * Evaluate the left and right subtrees if requested and then
 	 * convert their value into the requested type.
 	 */
-	if (lefttype != VALUE_NONE)
+	if (leftType != VALUE_NONE)
 	{
-		leftval = EvaluateNode(tree, left);
+		leftVal = EvaluateNode(tree, left);
 
-		if (!ConvertType(leftval, lefttype, &leftval))
-			return badvalue;
+		if (!ConvertType(leftVal, leftType, &leftVal))
+			return badValue;
 	}
 
-	if (righttype != VALUE_NONE)
+	if (rightType != VALUE_NONE)
 	{
-		rightval = EvaluateNode(tree, right);
+		rightVal = EvaluateNode(tree, right);
 
-		if (!ConvertType(rightval, righttype, &rightval))
-			return badvalue;
+		if (!ConvertType(rightVal, rightType, &rightVal))
+			return badValue;
 	}
 
-	value.type = valtype;
+	value.type = valType;
 
 	/*
 	 * Now operate on the converted values.
@@ -388,65 +423,65 @@ EvaluateNode(TREE * tree, NODE * node)
 	switch (node->op)
 	{
 		case OP_AND:
-			value.intval = leftval.intval & rightval.intval;
+			value.intVal = leftVal.intVal & rightVal.intVal;
 			break;
 
 		case OP_OR:
-			value.intval = leftval.intval | rightval.intval;
+			value.intVal = leftVal.intVal | rightVal.intVal;
 			break;
 
 		case OP_XOR:
-			value.intval = leftval.intval ^ rightval.intval;
+			value.intVal = leftVal.intVal ^ rightVal.intVal;
 			break;
 
 		case OP_COMPLEMENT:
-			value.intval = ~leftval.intval;
+			value.intVal = ~leftVal.intVal;
 			break;
 
 		case OP_NEGATE:
-			value.intval = -leftval.intval;
+			value.intVal = -leftVal.intVal;
 			break;
 
 		case OP_ADD:
-			value.intval = leftval.intval + rightval.intval;
+			value.intVal = leftVal.intVal + rightVal.intVal;
 			break;
 
 		case OP_SUBTRACT:
-			value.intval = leftval.intval - rightval.intval;
+			value.intVal = leftVal.intVal - rightVal.intVal;
 			break;
 
 		case OP_MULTIPLY:
-			value.intval = leftval.intval * rightval.intval;
+			value.intVal = leftVal.intVal * rightVal.intVal;
 			break;
 
 		case OP_DIVIDE:
-			if (rightval.intval == 0)
-				return badvalue;
+			if (rightVal.intVal == 0)
+				return badValue;
 
-			value.intval = leftval.intval / rightval.intval;
+			value.intVal = leftVal.intVal / rightVal.intVal;
 			break;
 
 		case OP_MODULO:
-			if (rightval.intval == 0)
-				return badvalue;
+			if (rightVal.intVal == 0)
+				return badValue;
 
-			value.intval = leftval.intval % rightval.intval;
+			value.intVal = leftVal.intVal % rightVal.intVal;
 			break;
 
 		case OP_NOT:
-			value.intval = !leftval.intval;
+			value.intVal = !leftVal.intVal;
 			break;
 
 		case OP_ANDAND:
-			value.intval = leftval.intval && rightval.intval;
+			value.intVal = leftVal.intVal && rightVal.intVal;
 			break;
 
 		case OP_OROR:
-			value.intval = leftval.intval || rightval.intval;
+			value.intVal = leftVal.intVal || rightVal.intVal;
 			break;
 
 		case OP_ALTERNATION:
-			if (leftval.intval != 0)
+			if (leftVal.intVal != 0)
 				value = EvaluateNode(tree, right);
 			else
 				value = EvaluateNode(tree, node->child[2]);
@@ -454,7 +489,7 @@ EvaluateNode(TREE * tree, NODE * node)
 			break;
 
 		default:
-			return badvalue;
+			return badValue;
 	}
 
 	return value;
@@ -465,40 +500,40 @@ EvaluateNode(TREE * tree, NODE * node)
  * Evaluate a function call.
  */
 static VALUE
-EvaluateFunction(TREE * tree, NODE * node)
+EvaluateFunction(TREE * tree, const NODE * node)
 {
 	VALUE	value;
 	VALUE	arg1;
 	VALUE	arg2;
 
 	if (node->op != OP_FUNCTION)
-		return badvalue;
+		return badValue;
 
 	/*
 	 * Check for the magic "my" function, which acts specially.
 	 * We cannot evaluate its argument before calling it since
 	 * its argument is special.
 	 */
-	if (node->intval == FUNC_MY)
+	if (node->intVal == FUNC_MY)
 		return FunctionMy(tree, node->child[0]);
 
 	/*
 	 * Evaluate the number of arguments indicated by the function id.
 	 */
-	switch (FUNCTION_ARGS(node->intval))
+	switch (FUNCTION_ARGS(node->intVal))
 	{
 		case 2:	arg2 = EvaluateNode(tree, node->child[1]);
 		case 1: arg1 = EvaluateNode(tree, node->child[0]);
 		case 0: break;
 
 		default:
-			return badvalue;
+			return badValue;
 	}
 
 	/*
 	 * Call the proper function indicated by the function id.
 	 */
-	switch (node->intval)
+	switch (node->intVal)
 	{
 		case FUNC_MATCH:
 			return FunctionMatch(tree, arg1, arg2);
@@ -522,7 +557,7 @@ EvaluateFunction(TREE * tree, NODE * node)
 			return FunctionCmp(tree, arg1, arg2);
 
 		default:
-			return badvalue;
+			return badValue;
 	}
 
 	return value;
@@ -545,29 +580,29 @@ FunctionMatch(TREE * tree, VALUE arg1, VALUE arg2)
 	 */
 	if ((arg1.type == VALUE_BOOLEAN) || (arg1.type == VALUE_NUMBER))
 	{
-		sprintf(buf1, "%ld", arg1.intval);
+		sprintf(buf1, "%ld", arg1.intVal);
 		arg1.type = VALUE_STRING;
-		arg1.strval = buf1;
+		arg1.strVal = buf1;
 	}
 
 	if ((arg2.type == VALUE_BOOLEAN) || (arg2.type == VALUE_NUMBER))
 	{
-		sprintf(buf2, "%ld", arg2.intval);
+		sprintf(buf2, "%ld", arg2.intVal);
 		arg2.type = VALUE_STRING;
-		arg2.strval = buf2;
+		arg2.strVal = buf2;
 	}
 
 	/*
 	 * If they are not both strings now, then return a bad value.
 	 */
 	if ((arg1.type != VALUE_STRING) || (arg2.type != VALUE_STRING))
-		return badvalue;
+		return badValue;
 
 	/*
 	 * They are both strings now so do the pattern match.
 	 */
 	value.type = VALUE_BOOLEAN;
-	value.intval = PatternMatch(arg1.strval, arg2.strval);
+	value.intVal = PatternMatch(arg1.strVal, arg2.strVal);
 
 	return value;
 }
@@ -588,11 +623,11 @@ FunctionMin(TREE * tree, VALUE arg1, VALUE arg2)
 	if (((arg1.type != VALUE_BOOLEAN) && (arg1.type != VALUE_NUMBER)) ||
 		((arg2.type != VALUE_BOOLEAN) && (arg2.type != VALUE_NUMBER)))
 	{
-		return badvalue;
+		return badValue;
 	}
 
 	value.type = VALUE_NUMBER;
-	value.intval = (arg1.intval < arg2.intval) ? arg1.intval : arg2.intval;
+	value.intVal = (arg1.intVal < arg2.intVal) ? arg1.intVal : arg2.intVal;
 
 	return value;
 }
@@ -613,11 +648,11 @@ FunctionMax(TREE * tree, VALUE arg1, VALUE arg2)
 	if (((arg1.type != VALUE_BOOLEAN) && (arg1.type != VALUE_NUMBER)) ||
 		((arg2.type != VALUE_BOOLEAN) && (arg2.type != VALUE_NUMBER)))
 	{
-		return badvalue;
+		return badValue;
 	}
 
 	value.type = VALUE_NUMBER;
-	value.intval = (arg1.intval > arg2.intval) ? arg1.intval : arg2.intval;
+	value.intVal = (arg1.intVal > arg2.intVal) ? arg1.intVal : arg2.intVal;
 
 	return value;
 }
@@ -633,10 +668,10 @@ FunctionAbs(TREE * tree, VALUE arg)
 	VALUE	value;
 
 	if ((arg.type != VALUE_BOOLEAN) && (arg.type != VALUE_NUMBER))
-		return badvalue;
+		return badValue;
 
 	value.type = VALUE_NUMBER;
-	value.intval = (arg.intval < 0) ? -arg.intval : arg.intval;
+	value.intVal = (arg.intVal < 0) ? -arg.intVal : arg.intVal;
 
 	return value;
 }
@@ -655,7 +690,7 @@ FunctionStr(TREE * tree, VALUE arg)
 		return arg;
 
 	if ((arg.type != VALUE_BOOLEAN) && (arg.type != VALUE_NUMBER))
-		return badvalue;
+		return badValue;
 
 	value.type = VALUE_STRING;
 
@@ -663,16 +698,16 @@ FunctionStr(TREE * tree, VALUE arg)
 	 * Check a few common values specially, and if it is not one
 	 * of those, then allocate a temporary string for the value.
 	 */
-	if (arg.intval == 0)
-		value.strval = "0";
-	else if (arg.intval == 1)
-		value.strval = "1";
-	else if (arg.intval == -1)
-		value.strval = "-1";
+	if (arg.intVal == 0)
+		value.strVal = "0";
+	else if (arg.intVal == 1)
+		value.strVal = "1";
+	else if (arg.intVal == -1)
+		value.strVal = "-1";
 	else
 	{
-		sprintf(buf, "%ld", arg.intval);
-		value.strval = CopyTempString(buf);
+		sprintf(buf, "%ld", arg.intVal);
+		value.strVal = CopyTempString(buf);
 	}
 
 	return value;
@@ -693,22 +728,22 @@ FunctionStrlen(TREE * tree, VALUE arg)
 	 */
 	if ((arg.type == VALUE_BOOLEAN) || (arg.type == VALUE_NUMBER))
 	{
-		sprintf(buf, "%ld", arg.intval);
+		sprintf(buf, "%ld", arg.intVal);
 		arg.type = VALUE_STRING;
-		arg.strval = buf;
+		arg.strVal = buf;
 	}
 
 	/*
 	 * If it is not a string now, then return a bad value.
 	 */
 	if ((arg.type != VALUE_STRING))
-		return badvalue;
+		return badValue;
 
 	/*
 	 * Return the length of the string.
 	 */
 	value.type = VALUE_NUMBER;
-	value.intval = strlen(arg.strval);
+	value.intVal = strlen(arg.strVal);
 
 	return value;
 }
@@ -732,13 +767,13 @@ FunctionCmp(TREE * tree, VALUE arg1, VALUE arg2)
 		((arg2.type == VALUE_BOOLEAN) || (arg2.type == VALUE_NUMBER)))
 	{
 		value.type = VALUE_NUMBER;
-		value.intval = 0;
+		value.intVal = 0;
 
-		if (arg1.intval < arg2.intval)
-			value.intval = -1;
+		if (arg1.intVal < arg2.intVal)
+			value.intVal = -1;
 
-		if (arg1.intval > arg2.intval)
-			value.intval = 1;
+		if (arg1.intVal > arg2.intVal)
+			value.intVal = 1;
 
 		return value;
 	}
@@ -748,35 +783,35 @@ FunctionCmp(TREE * tree, VALUE arg1, VALUE arg2)
 	 */
 	if ((arg1.type == VALUE_BOOLEAN) || (arg1.type == VALUE_NUMBER))
 	{
-		sprintf(buf1, "%ld", arg1.intval);
+		sprintf(buf1, "%ld", arg1.intVal);
 		arg1.type = VALUE_STRING;
-		arg1.strval = buf1;
+		arg1.strVal = buf1;
 	}
 
 	if ((arg2.type == VALUE_BOOLEAN) || (arg2.type == VALUE_NUMBER))
 	{
-		sprintf(buf2, "%ld", arg2.intval);
+		sprintf(buf2, "%ld", arg2.intVal);
 		arg2.type = VALUE_STRING;
-		arg2.strval = buf2;
+		arg2.strVal = buf2;
 	}
 
 	/*
 	 * Make sure the arguments are strings now.
 	 */
 	if ((arg1.type != VALUE_STRING) || (arg2.type != VALUE_STRING))
-		return badvalue;
+		return badValue;
 
 	/*
 	 * OK, now do a string compare and normalize the result.
 	 */
 	value.type = VALUE_NUMBER;
-	value.intval = strcmp(arg1.strval, arg2.strval);
+	value.intVal = strcmp(arg1.strVal, arg2.strVal);
 
-	if (value.intval < 0)
-		value.intval = -1;
+	if (value.intVal < 0)
+		value.intVal = -1;
 
-	if (value.intval > 0)
-		value.intval = 1;
+	if (value.intVal > 0)
+		value.intVal = 1;
 
 	return value;
 }
@@ -788,12 +823,12 @@ FunctionCmp(TREE * tree, VALUE arg1, VALUE arg2)
  * This function is special and doesn't work like the other functions.
  */
 static VALUE
-FunctionMy(TREE * tree, NODE * node)
+FunctionMy(TREE * tree, const NODE * node)
 {
-	const PROC *	saved_proc;
+	const PROC *	savedProc;
 	PROC *		proc;
-	int		saved_uid;
-	int		saved_gid;
+	int		savedUid;
+	int		savedGid;
 	VALUE		value;
 
 	/*
@@ -802,15 +837,15 @@ FunctionMy(TREE * tree, NODE * node)
 	 * Save our effective user and group ids and replace them with our
 	 * real ones.
 	 */
-	proc = FindProcess(my_pid);
+	proc = FindProcess(myPid, NO_THREAD_ID);
 
-	saved_uid = proc->uid;
-	saved_gid = proc->gid;
+	savedUid = proc->uid;
+	savedGid = proc->gid;
 
-	proc->uid = my_uid;
-	proc->gid = my_gid;
+	proc->uid = myUid;
+	proc->gid = myGid;
 
-	saved_proc = tree->proc;
+	savedProc = tree->proc;
 	tree->proc = proc;
 
 	/*
@@ -823,10 +858,10 @@ FunctionMy(TREE * tree, NODE * node)
 	 * Restore our effective user and group ids, then replace the
 	 * process pointer in the tree to what it should be.
 	 */
-	proc->uid = saved_uid;
-	proc->gid = saved_gid;
+	proc->uid = savedUid;
+	proc->gid = savedGid;
 
-	tree->proc = saved_proc;
+	tree->proc = savedProc;
 
 	return value;
 }
@@ -838,25 +873,25 @@ FunctionMy(TREE * tree, NODE * node)
  * the bad value if the comparison is illegal.
  */
 static VALUE
-EvaluateComparison(TREE * tree, NODE * node)
+EvaluateComparison(TREE * tree, const NODE * node)
 {
 	int	compare;
-	VALUE	leftval;
-	VALUE	rightval;
+	VALUE	leftVal;
+	VALUE	rightVal;
 	VALUE	value;
 
-	leftval = EvaluateNode(tree, node->child[0]);
+	leftVal = EvaluateNode(tree, node->child[0]);
 
-	if (leftval.type == VALUE_BAD)
-		return badvalue;
+	if (leftVal.type == VALUE_BAD)
+		return badValue;
 
-	rightval = EvaluateNode(tree, node->child[1]);
+	rightVal = EvaluateNode(tree, node->child[1]);
 
-	if (rightval.type == VALUE_BAD)
-		return badvalue;
+	if (rightVal.type == VALUE_BAD)
+		return badValue;
 
-	if (!CompareValues(leftval, rightval, &compare))
-		return badvalue;
+	if (!CompareValues(leftVal, rightVal, &compare))
+		return badValue;
 
 	/*
 	 * Now set the result according to the comparison required
@@ -867,31 +902,31 @@ EvaluateComparison(TREE * tree, NODE * node)
 	switch (node->op)
 	{
 		case OP_EQUAL:
-			value.intval = (compare == 0);
+			value.intVal = (compare == 0);
 			break;
 
 		case OP_NOTEQUAL:
-			value.intval = (compare != 0);
+			value.intVal = (compare != 0);
 			break;
 
 		case OP_LESS:
-			value.intval = (compare < 0);
+			value.intVal = (compare < 0);
 			break;
 
 		case OP_LESSEQUAL:
-			value.intval = (compare <= 0);
+			value.intVal = (compare <= 0);
 			break;
 
 		case OP_GREATER:
-			value.intval = (compare > 0);
+			value.intVal = (compare > 0);
 			break;
 
 		case OP_GREATEREQUAL:
-			value.intval = (compare >= 0);
+			value.intVal = (compare >= 0);
 			break;
 
 		default:
-			return badvalue;
+			return badValue;
 	}
 
 	return value;
@@ -905,28 +940,26 @@ EvaluateComparison(TREE * tree, NODE * node)
  * Returns TRUE if the comparison is valid.
  */
 BOOL
-CompareValues(const VALUE leftval, const VALUE rightval, int * result)
+CompareValues(const VALUE leftVal, const VALUE rightVal, int * result)
 {
-	int	compare;
+	int	compare = 0;
 
-	compare = 0;
-
-	switch (TWOVAL(leftval.type, rightval.type))
+	switch (TWOVAL(leftVal.type, rightVal.type))
 	{
 		case TWOVAL(VALUE_NUMBER, VALUE_NUMBER):
 		case TWOVAL(VALUE_NUMBER, VALUE_BOOLEAN):
 		case TWOVAL(VALUE_BOOLEAN, VALUE_NUMBER):
 		case TWOVAL(VALUE_BOOLEAN, VALUE_BOOLEAN):
-			if (leftval.intval < rightval.intval)
+			if (leftVal.intVal < rightVal.intVal)
 				compare = -1;
 
-			if (leftval.intval > rightval.intval)
+			if (leftVal.intVal > rightVal.intVal)
 				compare = 1;
 
 			break;
 
 		case TWOVAL(VALUE_STRING, VALUE_STRING):
-			compare = strcmp(leftval.strval, rightval.strval);
+			compare = strcmp(leftVal.strVal, rightVal.strVal);
 
 			break;;
 
@@ -948,8 +981,8 @@ static BOOL
 ConvertType(VALUE val, int type, VALUE * retval)
 {
 	const char *	cp;
-	long		intval;
-	BOOL		neg;
+	long		intVal;
+	BOOL		isNeg;
 
 	/*
 	 * If the type is already correct, then just return the same value.
@@ -969,64 +1002,64 @@ ConvertType(VALUE val, int type, VALUE * retval)
 	{
 		case TWOVAL(VALUE_STRING, VALUE_BOOLEAN):
 			retval->type = VALUE_BOOLEAN;
-			retval->intval = (val.strval[0] != '\0');
+			retval->intVal = (val.strVal[0] != '\0');
 			break;
 
 		case TWOVAL(VALUE_NUMBER, VALUE_BOOLEAN):
 			retval->type = VALUE_BOOLEAN;
-			retval->intval = (val.intval != 0);
+			retval->intVal = (val.intVal != 0);
 			break;
 
 		case TWOVAL(VALUE_BOOLEAN, VALUE_NUMBER):
 			retval->type = VALUE_NUMBER;
-			retval->intval = val.intval;
+			retval->intVal = val.intVal;
 			break;
 
 		case TWOVAL(VALUE_STRING, VALUE_NUMBER):
-			cp = val.strval;
+			cp = val.strVal;
 
-			intval = 0;
-			neg = FALSE;
+			intVal = 0;
+			isNeg = FALSE;
 
-			while (isblank(*cp))
+			while (isBlank(*cp))
 				cp++;
 
 			if (*cp == '-')
 			{
 				cp++;
-				neg = TRUE;
+				isNeg = TRUE;
 			}
 
-			if (!isdigit(*cp))
+			if (!isDigit(*cp))
 			{
-				*retval = badvalue;
+				*retval = badValue;
 
 				return FALSE;
 			}
 
-			while (isdigit(*cp))
-				intval = intval * 10 + *cp++ - '0';
+			while (isDigit(*cp))
+				intVal = intVal * 10 + *cp++ - '0';
 
-			while (isblank(*cp))
+			while (isBlank(*cp))
 				cp++;
 
 			if (*cp)
 			{
-				*retval = badvalue;
+				*retval = badValue;
 
 				return FALSE;
 			}
 
-			if (neg)
-				intval = -intval;
+			if (isNeg)
+				intVal = -intVal;
 
-			retval->intval = intval;
+			retval->intVal = intVal;
 			retval->type = VALUE_NUMBER;
 
 			break;
 
 		default:
-			*retval = badvalue;
+			*retval = badValue;
 
 			return FALSE;
 	}
@@ -1042,7 +1075,7 @@ ConvertType(VALUE val, int type, VALUE * retval)
 void
 ClearCondition(void)
 {
-	cond_flag = FALSE;
+	condFlag = FALSE;
 }
 
 
@@ -1054,15 +1087,15 @@ ClearCondition(void)
  * or NULL.  Example: "percent-cpu || state == 'R'"
  */
 BOOL
-ParseCondition(char * str)
+ParseCondition(const char * str)
 {
-	TREE	treedata;
+	TREE	treeData;
 
-	if (!ParseTree(&treedata, str, 0))
+	if (!ParseTree(&treeData, str, 0))
 		return FALSE;
 
-	cond_tree = treedata;
-	cond_flag = TRUE;
+	condTree = treeData;
+	condFlag = TRUE;
 
 	return TRUE;
 }
@@ -1074,21 +1107,21 @@ ParseCondition(char * str)
  * The depth shows the recursion depth of expression macros.
  */
 BOOL
-ParseTree(TREE * tree, char * str, int depth)
+ParseTree(TREE * tree, const char * str, int depth)
 {
 	int	len;
 
-	badvalue.type = VALUE_BAD;
-	badvalue.strval = "";
-	badvalue.intval = 0;
-	badvalue.column = NULL;
+	badValue.type = VALUE_BAD;
+	badValue.strVal = "";
+	badValue.intVal = 0;
+	badValue.column = NULL;
 
 	len = strlen(str);
 
 	tree->expr = malloc(len + 1);
-	tree->modexpr = malloc(len + 1);;
+	tree->modExpr = malloc(len + 1);;
 
-	if ((tree->expr == NULL) || (tree->modexpr == NULL))
+	if ((tree->expr == NULL) || (tree->modExpr == NULL))
 	{
 		fprintf(stderr, "Cannot allocate condition expression\n");
 
@@ -1096,15 +1129,15 @@ ParseTree(TREE * tree, char * str, int depth)
 	}
 
 	memcpy(tree->expr, str, len + 1);
-	memcpy(tree->modexpr, str, len + 1);
+	memcpy(tree->modExpr, str, len + 1);
 
 	tree->cp = tree->expr;
 	tree->failed = FALSE;
 	tree->root = NULL;
 	tree->token = TOKEN_BAD;
-	tree->rescantoken = FALSE;
-	tree->tokenstr = "";
-	tree->tokenint = 0;
+	tree->rescanToken = FALSE;
+	tree->tokenStr = "";
+	tree->tokenInt = 0;
 	tree->depth = depth;
 
 	/*
@@ -1136,7 +1169,7 @@ ParseAlternation(TREE * tree)
 {
 	NODE *	top;
 	NODE *	right;
-	NODE *	altright;
+	NODE *	altRight;
 	TOKEN	token;
 
 	top = ParseOrOr(tree);
@@ -1145,7 +1178,7 @@ ParseAlternation(TREE * tree)
 
 	if (token != TOKEN_QUESTIONMARK)
 	{
-		tree->rescantoken = TRUE;
+		tree->rescanToken = TRUE;
 
 		return top;
 	}
@@ -1156,16 +1189,16 @@ ParseAlternation(TREE * tree)
 
 	if (token != TOKEN_COLON)
 	{
-		tree->rescantoken = TRUE;
+		tree->rescanToken = TRUE;
 
 		return BadNode(tree, "Missing colon in conditional");
 	}
 
-	altright = ParseOrOr(tree);
+	altRight = ParseOrOr(tree);
 
 	top = NewNode(tree, OP_OROR, top, right);
 
-	top->child[2] = altright;
+	top->child[2] = altRight;
 
 	return top;
 }
@@ -1189,7 +1222,7 @@ ParseOrOr(TREE * tree)
 
 		if (token != TOKEN_OROR)
 		{
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return top;
 		}
@@ -1219,7 +1252,7 @@ ParseAndAnd(TREE * tree)
 
 		if (token != TOKEN_ANDAND)
 		{
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return top;
 		}
@@ -1273,7 +1306,7 @@ ParseRelation(TREE * tree)
 			break;
 
 		default:
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return top;
 	}
@@ -1315,7 +1348,7 @@ ParseSum(TREE * tree)
 				break;
 
 			default:
-				tree->rescantoken = TRUE;
+				tree->rescanToken = TRUE;
 
 				return top;
 		}
@@ -1361,7 +1394,7 @@ ParseProduct(TREE * tree)
 				break;
 
 			default:
-				tree->rescantoken = TRUE;
+				tree->rescanToken = TRUE;
 
 				return top;
 		}
@@ -1392,7 +1425,7 @@ ParseOr(TREE * tree)
 
 		if (token != TOKEN_OR)
 		{
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return top;
 		}
@@ -1434,7 +1467,7 @@ ParseAnd(TREE * tree)
 				break;
 
 			default:
-				tree->rescantoken = TRUE;
+				tree->rescanToken = TRUE;
 
 				return top;
 		}
@@ -1467,7 +1500,7 @@ ParseShift(TREE * tree)
 			break;
 
 		default:
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return top;
 	}
@@ -1495,7 +1528,7 @@ ParseUnary(TREE * tree)
 				ParseUnary(tree), NULL);
 
 		default:
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return ParseTerm(tree);
 	}
@@ -1516,19 +1549,19 @@ ParseTerm(TREE * tree)
 	switch (token)
 	{
 		case TOKEN_SYMBOL:
-			return ParseSymbol(tree, tree->tokenstr);
+			return ParseSymbol(tree, tree->tokenStr);
 
 		case TOKEN_STRING:
 			node = NewNode(tree, OP_STRING, NULL, NULL);
 
-			node->strval = tree->tokenstr;
+			node->strVal = tree->tokenStr;
 
 			return node;
 
 		case TOKEN_NUMBER:
 			node = NewNode(tree, OP_NUMBER, NULL, NULL);
 
-			node->intval = tree->tokenint;
+			node->intVal = tree->tokenInt;
 
 			return node;
 
@@ -1575,7 +1608,7 @@ ParseSymbol(TREE * tree, char * str)
 	 */
 	if (token == TOKEN_LEFTPAREN)
 	{
-		tree->rescantoken = TRUE;
+		tree->rescanToken = TRUE;
 
 		return ParseFunction(tree, str);
 	}
@@ -1584,9 +1617,9 @@ ParseSymbol(TREE * tree, char * str)
 	 * See if the symbol name is upper case.
 	 * If so, then it is a macro that needs expanding.
 	 */
-	if (isupper(*str))
+	if (isUpper(*str))
 	{
-		tree->rescantoken = TRUE;
+		tree->rescanToken = TRUE;
 
 		return ParseMacro(tree, str);
 	}
@@ -1615,7 +1648,7 @@ ParseSymbol(TREE * tree, char * str)
 	 */
 	if (token != TOKEN_PERIOD)
 	{
-		tree->rescantoken = TRUE;
+		tree->rescanToken = TRUE;
 
 		node = NewNode(tree, OP_COLUMN_BASE, NULL, NULL);
 
@@ -1634,11 +1667,11 @@ ParseSymbol(TREE * tree, char * str)
 	if (token != TOKEN_SYMBOL)
 		return BadNode(tree, "Missing column qualifier");
 
-	if (strcmp(tree->tokenstr, "base") == 0)
+	if (strcmp(tree->tokenStr, "base") == 0)
 		op = OP_COLUMN_BASE;
-	else if (strcmp(tree->tokenstr, "show") == 0)
+	else if (strcmp(tree->tokenStr, "show") == 0)
 		op = OP_COLUMN_SHOW;
-	else if (strcmp(tree->tokenstr, "test") == 0)
+	else if (strcmp(tree->tokenStr, "test") == 0)
 		op = OP_COLUMN_TEST;
 	else
 	{
@@ -1665,7 +1698,7 @@ ParseFunction(TREE * tree, char * str)
 {
 	NODE *	node;
 	int	func;
-	int	argcount;
+	int	argCount;
 	int	i;
 
 	func = 0;
@@ -1714,21 +1747,21 @@ ParseFunction(TREE * tree, char * str)
 
 	node = NewNode(tree, OP_FUNCTION, NULL, NULL);
 
-	node->intval = func;
+	node->intVal = func;
 
 	/*
 	 * Collect the arguments and attach them to the function node.
 	 */
-	argcount = FUNCTION_ARGS(func);
+	argCount = FUNCTION_ARGS(func);
 
-	if (argcount > CHILDS)
+	if (argCount > CHILDS)
 		return BadNode(tree, "Too many function arguments");
 
-	for (i = 0; i < argcount; i++)
+	for (i = 0; i < argCount; i++)
 	{
 		if ((i > 0) && (ParseToken(tree) != TOKEN_COMMA))
 		{
-			tree->rescantoken = TRUE;
+			tree->rescanToken = TRUE;
 
 			return BadNode(tree,
 				"Not enough arguments for function");
@@ -1781,7 +1814,7 @@ ParseFunction(TREE * tree, char * str)
 static NODE *
 ParseMacro(TREE * tree, char * str)
 {
-	TREE	subtree;
+	TREE	subTree;
 	ARGS	args;
 
 	if (tree->depth >= MAX_EXPR_DEPTH)
@@ -1797,14 +1830,14 @@ ParseMacro(TREE * tree, char * str)
 	 * Parse the expression.  If it fails, an error message has
 	 * already been given so we don't need to add another one.
 	 */
-	if (!ParseTree(&subtree, args.table[0], tree->depth + 1))
+	if (!ParseTree(&subTree, args.table[0], tree->depth + 1))
 	{
 		tree->failed = TRUE;
 
 		return BadNode(tree, "");
 	}
 
-	return subtree.root;	
+	return subTree.root;	
 }
 
 
@@ -1815,10 +1848,10 @@ ParseMacro(TREE * tree, char * str)
 static TOKEN
 ParseToken(TREE * tree)
 {
-	if (!tree->rescantoken)
+	if (!tree->rescanToken)
 		tree->token = ParseNewToken(tree);
 
-	tree->rescantoken = FALSE;
+	tree->rescanToken = FALSE;
 
 	return tree->token;
 }
@@ -1832,11 +1865,11 @@ ParseNewToken(TREE * tree)
 {
 	int	ch;
 
-	while (isblank(*tree->cp))
+	while (isBlank(*tree->cp))
 		tree->cp++;
 
-	tree->tokenstr = &tree->modexpr[tree->cp - tree->expr];
-	tree->tokenint = 0;
+	tree->tokenStr = &tree->modExpr[tree->cp - tree->expr];
+	tree->tokenInt = 0;
 
 	/*
 	 * Switch on the type of character.
@@ -1951,7 +1984,7 @@ ParseNewToken(TREE * tree)
 
 		case '\'':
 		case '"':
-			tree->tokenstr++;
+			tree->tokenStr++;
 
 			while (*tree->cp != ch)
 			{
@@ -1964,7 +1997,7 @@ ParseNewToken(TREE * tree)
 				return BadToken(tree, "Unterminated string");
 			}
 
-			tree->modexpr[tree->cp - tree->expr] = '\0';
+			tree->modExpr[tree->cp - tree->expr] = '\0';
 
 			tree->cp++;
 
@@ -1977,7 +2010,7 @@ ParseNewToken(TREE * tree)
 	/*
 	 * Check for a numeric value.
 	 */
-	if (isdigit(ch))
+	if (isDigit(ch))
 	{
 		tree->cp--;
 		return ParseNumber(tree);
@@ -1986,17 +2019,17 @@ ParseNewToken(TREE * tree)
 	/*
 	 * We must now have a symbol name.
 	 */
-	if (!isbegsymbol(ch))
+	if (!isBegSymbol(ch))
 		return BadToken(tree, "Illegal token");
 
 	/*
 	 * Yes, it is a symbol, so see how large it is and null terminate it.
 	 */
-	while (issymbol(ch))
+	while (isSymbol(ch))
 		ch = *tree->cp++;
 
 	tree->cp--;
-	tree->modexpr[tree->cp - tree->expr] = '\0';
+	tree->modExpr[tree->cp - tree->expr] = '\0';
 
 	return TOKEN_SYMBOL;
 }
@@ -2017,7 +2050,7 @@ ParseNumber(TREE * tree)
 	value = 0;
 	ch = *tree->cp++;
 
-	if (!isdigit(ch))
+	if (!isDigit(ch))
 	{
 		tree->cp--;
 
@@ -2029,7 +2062,7 @@ ParseNumber(TREE * tree)
 	 */
 	if (ch != '0')
 	{
-		while (isdigit(ch))
+		while (isDigit(ch))
 		{
 			value = value * 10 + ch - '0';
 			ch = *tree->cp++;
@@ -2037,10 +2070,10 @@ ParseNumber(TREE * tree)
 
 		tree->cp--;
 
-		if (issymbol(ch))
+		if (isSymbol(ch))
 			return BadToken(tree, "Bad decimal number");
 
-		tree->tokenint = value;
+		tree->tokenInt = value;
 
 		return TOKEN_NUMBER;
 	}
@@ -2053,7 +2086,7 @@ ParseNumber(TREE * tree)
 
 	if ((ch != 'x') && (ch != 'X'))
 	{
-		while (isoctal(ch))
+		while (isOctal(ch))
 		{
 			value = value * 8 + ch - '0';
 			ch = *tree->cp++;
@@ -2061,10 +2094,10 @@ ParseNumber(TREE * tree)
 
 		tree->cp--;
 
-		if (issymbol(ch))
+		if (isSymbol(ch))
 			return BadToken(tree, "Bad octal number");
 
-		tree->tokenint = value;
+		tree->tokenInt = value;
 
 		return TOKEN_NUMBER;
 	}
@@ -2074,13 +2107,13 @@ ParseNumber(TREE * tree)
 	 */
 	ch = *tree->cp++;
 
-	while (ishex(ch))
+	while (isHex(ch))
 	{
 		value = value * 16;
 
-		if (isdigit(ch))
+		if (isDigit(ch))
 			value += (ch - '0');
-		else if (islower(ch))
+		else if (isLower(ch))
 			value += (ch - 'a' + 10);
 		else
 			value += (ch - 'A' + 10);
@@ -2090,10 +2123,10 @@ ParseNumber(TREE * tree)
 
 	tree->cp--;
 
-	if (issymbol(ch))
+	if (isSymbol(ch))
 		return BadToken(tree, "Bad hex number");
 
-	tree->tokenint = value;
+	tree->tokenInt = value;
 
 	return TOKEN_NUMBER;
 }
@@ -2139,8 +2172,8 @@ NewNode(TREE * tree, OP op, NODE * left, NODE * right)
 	node->child[1] = right;
 	node->child[2] = NULL;
 	node->column = NULL;
-	node->strval = "";
-	node->intval = 0;
+	node->strVal = "";
+	node->intVal = 0;
 
 	return node;
 }
@@ -2161,10 +2194,10 @@ BadNode(TREE * tree, char * message)
 		tree->failed = TRUE;
 	}
 
-	badnode.op = OP_NONE;
-	badnode.strval = "";
+	badNode.op = OP_NONE;
+	badNode.strVal = "";
 
-	return &badnode;
+	return &badNode;
 }
 
 /* END CODE */
