@@ -2,7 +2,7 @@
  * Configurable ps-like program.
  * Global definitions.
  *
- * Copyright (c) 2008 David I. Bell
+ * Copyright (c) 2010 David I. Bell
  * Permission is granted to use, distribute, or modify this source,
  * provided that this copyright notice remains intact.
  */
@@ -16,6 +16,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <time.h>
+#include <pthread.h>
 #include <sys/time.h>
 #include <sys/types.h>
 
@@ -42,12 +43,20 @@
 /*
  * Some constants related to the system.
  */
-#define	TICKS		100		/* ticks in a second */
 #define	CPU_SCALE	10000		/* scaling factor for cpu usage */
 #define	MEMORY_SCALE	1000		/* scaling factor for memory usage */
-#define	CLICK_SIZE	4096		/* size of memory allocation chunk */
 #define	BASE_USER_UID	100		/* first "user" uid */
 
+
+/*
+ * Constants for calculating CPU percentages.
+ */
+#define	PERCENT_RESOLUTION	5	/* max number of CPU samples per second */
+#define	PERCENT_MAX_SECONDS	20	/* max seconds for collecting CPU samples */
+#define	PERCENT_DEFAULT_SECONDS	10	/* default seconds for cpu percentage */
+#define	PERCENT_SAMPLES	((PERCENT_RESOLUTION) * (PERCENT_MAX_SECONDS) + 2)	/* total samples */
+
+
 /*
  * Define widths of numbers which depend on the size of a long.
  */
@@ -61,6 +70,7 @@
 #define	LONG_HEX_FORMAT		"%08lx"
 #endif
 
+
 /*
  * Some default values.
  */
@@ -80,14 +90,6 @@
 #define	DEFAULT_BACKGROUND	"white"	/* background color for X11 */
 
 
-/*
- * File descriptors.
- */
-#define	STDIN	0
-#define	STDOUT	1
-#define	STDERR	2
-
-
 /*
  * Some limits on the program.
  */
@@ -109,16 +111,19 @@
 #define	BUF_COMMAND_LEN	128
 #define	MAX_PROGRAM_LEN	32
 #define	MAX_WCHAN_LEN	32
+#define	MAX_STATES_LEN	16
+#define	MAX_COLORS	20
+#define	MAX_ROW_COLORS	50
 
 
 /*
  * Macros to help parse strings.
  */
-#define	isblank(ch)	(((ch) == ' ') || ((ch) == '\t'))
-#define	isdigit(ch)	(((ch) >= '0') && ((ch) <= '9'))
-#define	isupper(ch)	(((ch) >= 'A') && ((ch) <= 'Z'))
-#define	islower(ch)	(((ch) >= 'a') && ((ch) <= 'z'))
-#define	ismacro(ch)	isupper(ch)
+#define	isBlank(ch)	(((ch) == ' ') || ((ch) == '\t'))
+#define	isDigit(ch)	(((ch) >= '0') && ((ch) <= '9'))
+#define	isUpper(ch)	(((ch) >= 'A') && ((ch) <= 'Z'))
+#define	isLower(ch)	(((ch) >= 'a') && ((ch) <= 'z'))
+#define	isMacro(ch)	isUpper(ch)
 
 
 /*
@@ -158,86 +163,99 @@ typedef	struct
 
 /*
  * Structure which holds information about a process.
+ * This is used for main process information and also for threads
+ * making up the main process.  The main process entry is a summary
+ * of all the threads for a process and has a tid of NO_THREAD_ID.
+ * For a multi-threaded process each thread has its own entry,
+ * including the main starting thread.  Threads are linked together
+ * and know their main process owner entry.
  */
 typedef	struct	PROC	PROC;
 
 struct	PROC
 {
-	PROC *	next;			/* next process in list */
-	BOOL	isvalid;		/* the process is existent */
-	BOOL	isancient;		/* the process is before we started */
-	BOOL	isnew;			/* the process is just created */
-	BOOL	isactive;		/* the process is active */
-	BOOL	changed;		/* the process state has changed */
-	BOOL	isshown;		/* the process is to be shown */
-	BOOL	hascommand;		/* there is a real command line */
-	ULONG	live_counter;		/* counter to mark existant procs */
-	ULONG	run_order;		/* counter value when last active */
-	time_t	last_saved_time;	/* time status was copied to old */
-	time_t	last_active_time;	/* time process last was active */
-	time_t	last_sync_time;		/* time status was last synchonized */
+	PROC *	next;			/* next process or thread in list */
+	PROC *	nextThread;		/* first or next thread belonging to owner */
+	PROC *	owner;			/* main process owning this thread */
+	BOOL	isValid;		/* the process is existent */
+	BOOL	isAncient;		/* the process is before we started */
+	BOOL	isNew;			/* the process is just created */
+	BOOL	isActive;		/* the process is active */
+	BOOL	isChanged;		/* the process state has changed */
+	BOOL	isShown;		/* the process is to be shown */
+	BOOL	hasCommand;		/* there is a real command line */
+	BOOL	isThread;		/* this is a thread process entry */
+	ULONG	liveCounter;		/* counter to mark existant procs */
+	ULONG	runOrder;		/* counter value when last active */
+	time_t	lastSavedTime;		/* time status was copied to old */
+	time_t	lastActiveTime;		/* time process last was active */
+	time_t	lastSyncTime;		/* time status was last synchonized */
 	time_t	deathTime;		/* time process was seen to be dead */
 	uid_t	uid;			/* effective user id */
 	gid_t	gid;			/* effective group id */
 	pid_t	pid;			/* process id */
-	pid_t	parent_pid;		/* parent pid */
-	pid_t	process_group;		/* process group */
-	pid_t	session_id;		/* session id */
-	dev_t	tty_dev;		/* controlling terminal's device */
-	pid_t	tty_pgrp;		/* process group of terminal */
+	pthread_t	tid;		/* thread id (or NO_THREAD_ID) */
+	pid_t	parentPid;		/* parent pid */
+	pid_t	processGroup;		/* process group */
+	pid_t	sessionId;		/* session id */
+	dev_t	ttyDevice;		/* controlling terminal's device */
+	pid_t	ttyProcessGroup;	/* process group of terminal */
 	int	state;			/* process state character */
 	int	processor;		/* processor process is on */
-	int	exitsignal;		/* signal causing exit */
+	int	exitSignal;		/* signal causing exit */
 	ULONG	flags;			/* kernel flags */
-	long	min_flt;		/* minor page faults */
-	long	maj_flt;		/* major page faults */
-	long	cmin_flt;		/* child minor page faults */
-	long	cmaj_flt;		/* child major page faults */
-	long	utime;			/* user runtime in jiffies */
-	long	stime;			/* system runtime in jiffies */
-	long	cutime;			/* child user runtime */
-	long	cstime;			/* child system runtime */
+	long	minorFaults;		/* minor page faults */
+	long	majorFaults;		/* major page faults */
+	long	childMinorFaults;	/* child minor page faults */
+	long	childMajorFaults;	/* child major page faults */
+	long	userRunTime;		/* user runtime in jiffies */
+	long	systemRunTime;		/* system runtime in jiffies */
+	long	childUserRunTime;	/* child user runtime */
+	long	childSystemRunTime;	/* child system runtime */
 	long	nice;			/* nice value */
 	long	priority;		/* scheduling priority */
 	long	realTimePriority;	/* real time priority */
 	long	timeout;		/* timeout */
-	long	it_real_value;		/* jiffies sleeping for */
-	long	vsize;			/* virtual size of process in bytes */
+	long	itRealValue;		/* jiffies sleeping for */
+	long	virtualSize;		/* virtual size of process in bytes */
 	long	rss;			/* resident size in clicks */
-	long	rss_limit;		/* resident size limit */
+	long	rssLimit;		/* resident size limit */
 	long	policy;			/* policy */
-	ULONG	start_time_ticks;	/* jiffies uptime when started */
-	ULONG	start_time_clock;	/* time when started */
-	ULONG	start_code;		/* beginning address of code */
-	ULONG	end_code;		/* ending address of code */
-	ULONG	start_stack;		/* starting address of stack */
-	ULONG	esp;			/* spack pointer */
+	time_t	startTimeClock;		/* clock time when started */
+	ULONG	startTimeTicks;		/* jiffies uptime when started */
+	ULONG	firstCpuTime;		/* cpu runtime when first examined */
+	ULONG	startCode;		/* beginning address of code */
+	ULONG	endCode;		/* ending address of code */
+	ULONG	startStack;		/* starting address of stack */
+	ULONG	esp;			/* stack pointer */
 	ULONG	eip;			/* instruction pointer */
-	ULONG	wchan;			/* wait channel address */
+	ULONG	waitChan;		/* wait channel address */
 	ULONG	signal;			/* current signal */
-	ULONG	sigblock;		/* signals to block */
-	ULONG	sigignore;		/* signals to ignore */
-	ULONG	sigcatch;		/* signals to catch */
-	ULONG	nswap;			/* pages swapped out */
-	ULONG	cnswap;			/* child pages swapped out */
-	int	percent_cpu;		/* percentage of cpu used */
-	int	percent_memory;		/* percentage of memory used */
-	int	openfiles;		/* number of open files */
+	ULONG	sigBlock;		/* signals to block */
+	ULONG	sigIgnore;		/* signals to ignore */
+	ULONG	sigCatch;		/* signals to catch */
+	ULONG	pagesSwapped;		/* pages swapped out */
+	ULONG	childPagesSwapped;	/* child pages swapped out */
+	int	percentCpu;		/* percentage of cpu used */
+	int	percentMemory;		/* percentage of memory used */
+	int	openFiles;		/* number of open files */
 	int	threadCount;		/* number of threads */
-	int	command_length;		/* length of command line */
-	int	environment_length;	/* length of environment */
-	int	cwd_path_length;	/* length of current working dir path */
-	int	root_path_length;	/* length of root directory path */
-	int	exec_path_length;	/* length of executable path */
+	int	commandLength;		/* length of command line */
+	int	environmentLength;	/* length of environment */
+	int	cwdPathLength;		/* length of current working dir path */
+	int	rootPathLength;		/* length of root directory path */
+	int	execPathLength;		/* length of executable path */
 	char *	command;		/* command line */
 	char *	environment;		/* environment */
-	char *	cwd_path;		/* path of current working directory */
-	char *	root_path;		/* path of root directory */
-	char *	exec_path;		/* path of executable */
-	char *	stdio_paths[3];		/* paths of stdin, stdout, stderr */
+	char *	cwdPath;		/* path of current working directory */
+	char *	rootPath;		/* path of root directory */
+	char *	execPath;		/* path of executable */
+	char *	stdioPaths[3];		/* paths of stdin, stdout, stderr */
 	char	program[MAX_PROGRAM_LEN + 2];	/* program name */
-	char	command_buffer[BUF_COMMAND_LEN + 2];	/* command buffer */
-	char	wchan_symbol[MAX_WCHAN_LEN + 2];	/* wait channel */
+	char	commandBuffer[BUF_COMMAND_LEN + 2];	/* command buffer */
+	char	waitChanSymbol[MAX_WCHAN_LEN + 2];	/* wait channel */
+	char	states[MAX_STATES_LEN + 2];	/* thread states */
+	long	cpuTable[PERCENT_SAMPLES];	/* cpu runtime sample table */
 
 	/*
 	 * Status which is saved in order to determine active processes
@@ -245,22 +263,28 @@ struct	PROC
 	 * A process is active if any of this differs from the current
 	 * value.
 	 */
-	int	old_state;
-	int	old_flags;
-	long	old_min_flt;
-	long	old_maj_flt;
-	long	old_utime;
-	long	old_stime;
-	ULONG	old_start_time_ticks;
-	ULONG	old_end_code;	
-	ULONG	old_esp;
-	ULONG	old_eip;
-	ULONG	old_wchan;
+	int	oldState;
+	int	oldFlags;
+	long	oldMinorFaults;
+	long	oldMajorFaults;
+	long	oldUserRunTime;
+	long	oldSystemRunTime;
+	ULONG	oldStartTimeTicks;
+	ULONG	oldEndCode;	
+	ULONG	oldEsp;
+	ULONG	oldEip;
+	ULONG	oldWaitChan;
 };
 
 #define	NULL_PROC	((PROC *) 0)
 
 
+/*
+ * The thread id for the main process entry.
+ */
+#define	NO_THREAD_ID	((pthread_t) -1)
+
+
 /*
  * Column justification definitions.
  */
@@ -277,8 +301,8 @@ typedef	int	JUSTIFY;
 typedef	struct
 {
 	int		type;		/* type of value */
-	long		intval;		/* integer value */
-	const char *	strval;		/* string value */
+	long		intVal;		/* integer value */
+	const char *	strVal;		/* string value */
 	COLUMN *	column;		/* column value */
 } VALUE;
 
@@ -315,6 +339,7 @@ typedef	unsigned int	USEFLAG;
 #define	USE_USER_NAME	((USEFLAG) 0x1000)
 #define	USE_GROUP_NAME	((USEFLAG) 0x2000)
 #define	USE_WCHAN	((USEFLAG) 0x4000)
+#define	USE_THREADS	((USEFLAG) 0x8000)
 
 
 /*
@@ -324,15 +349,15 @@ struct	COLUMN
 {
 	char *		name;		/* column name for commands */
 	char *		heading;	/* heading string for column */
-	int		minwidth;	/* absolute minimum width of column */
-	int		initwidth;	/* initial minimum width of column */
+	int		minWidth;	/* absolute minimum width of column */
+	int		initWidth;	/* initial minimum width of column */
 	int		width;		/* actual minimum width of column */
 	JUSTIFY		justify;	/* how value is justified in column */
-	USEFLAG		useflag;	/* what data column uses */
-	const char *	(*showfunc)(const PROC * proc);
-	int		(*sortfunc)(const PROC * proc1, const PROC * proc2);
-	void		(*evalfunc)(const PROC * proc, VALUE * retval);
-	BOOL		(*testfunc)(const PROC * proc);
+	USEFLAG		useFlag;	/* what data column uses */
+	const char *	(*showFunc)(const PROC * proc);
+	int		(*sortFunc)(const PROC * proc1, const PROC * proc2);
+	void		(*evalFunc)(const PROC * proc, VALUE * retval);
+	BOOL		(*testFunc)(const PROC * proc);
 };
 
 
@@ -359,7 +384,10 @@ typedef	struct	DISPLAY	DISPLAY;
 struct	DISPLAY
 {
 	BOOL	(*open)(DISPLAY *);		/* open display */
+	BOOL	(*defineColor)(DISPLAY *, int, const char *, const char *, int);
+	void	(*createWindow)(DISPLAY *);	/* create window on display */
 	void	(*close)(DISPLAY *);		/* close display */
+	void	(*setColor)(DISPLAY *, int);	/* set color for output */
 	void	(*refresh)(DISPLAY *);		/* refresh display */
 	void	(*beginPage)(DISPLAY *);	/* begin output of page */
 	void	(*putChar)(DISPLAY *, int);	/* output character */
@@ -381,88 +409,118 @@ struct	DISPLAY
 #define	DISPLAY_TYPE_X11	"x11"
 
 
+/*
+ * Color related definitions.
+ */
+#define	DEFAULT_COLOR_ID	0
+#define	BAD_COLOR_ID		-1
+
+#define	DEFAULT_COLOR_NAME	"default"
+
+
+/*
+ * Flags which can be set in addition to the foreground and background
+ * to modify the text slightly.
+ */
+#define	COLOR_FLAG_NONE		0x00
+#define	COLOR_FLAG_BOLD		0x01
+#define	COLOR_FLAG_UNDERLINE	0x02
+
+
 /*
  * List of columns being shown.
  */
-extern	int	show_count;
-extern	COLUMN *show_list[MAX_COLUMNS];
+extern	int	showCount;
+extern	COLUMN *showList[MAX_COLUMNS];
 
 
 /*
  * Other global variables.
  */
-extern	ULONG		start_uptime;		/* uptime jiffies at start */
-extern	time_t		start_time;		/* clock time at start */
-extern	time_t		current_time;		/* current clock time */
-extern	long		total_memory_clicks;	/* amount of total memory */
-extern	ULONG		live_counter;		/* counter for live procs */
-extern	BOOL		ancient_flag;		/* seeing pre-existing procs */
-extern	BOOL		no_self;		/* don't show myself */
-extern	BOOL		no_root;		/* don't show root procs */
-extern	BOOL		no_header;		/* don't show column header */
-extern	BOOL		info;			/* show info line at top */
-extern	BOOL		my_procs;		/* only show my procs */
-extern	BOOL		active_only;		/* only show active procs */
-extern	BOOL		clear_screen;		/* clear screen each loop */
-extern	BOOL		loop;			/* loop showing status */
-extern	BOOL		running;		/* we still want to run */
-extern	BOOL		frozen;			/* data collection is frozen */
-extern	BOOL		update;			/* update once even if frozen */
-extern	BOOL		needRefresh;		/* need to refresh display */
-extern	BOOL		vertical;		/* vertical output format */
-extern	BOOL		top_set;		/* top option was used */
-extern	BOOL		top_auto;		/* autosize height for top */
-extern	BOOL		use_open_files;		/* using open file info */
-extern	BOOL		use_current_directory;	/* using current dir info */
-extern	BOOL		use_root_directory;	/* using root dir info */
-extern	BOOL		use_exec_inode;		/* using executable info */
-extern	BOOL		use_device_names;	/* using device name info */
-extern	BOOL		use_user_names;		/* using user name info */
-extern	BOOL		use_group_names;	/* using group name info */
-extern	BOOL		use_init;		/* using initial sleep */
-extern	BOOL		use_command;		/* using command line info */
-extern	BOOL		use_self;		/* using my own proc info */
-extern	BOOL		use_environment;	/* using environment info */
-extern	BOOL		use_wchan;		/* using wait channel symbol */
-extern	BOOL		use_stdio[3];		/* using various stdio info */
-extern	pid_t		my_pid;			/* my pid */
-extern	uid_t		my_uid;			/* my real user id */
-extern	gid_t		my_gid;			/* my real group id */
-extern	dev_t		null_dev;		/* device of /dev/null */
-extern	ino_t		null_inode;		/* inode of /dev/null */
-extern	int		proc_alloc_count;	/* allocated proc structures */
+extern	ULONG		startUptime;		/* uptime jiffies at start */
+extern	time_t		startTime;		/* clock time at start */
+extern	time_t		currentTime;		/* current clock time */
+extern	long		totalMemoryClicks;	/* amount of total memory */
+extern	long		ticksPerSecond;		/* number of clock ticks per second */
+extern	long		pageSize;		/* number of bytes in a page */
+extern	ULONG		liveCounter;		/* counter for live procs */
+extern	int		newCpuIndex;		/* new CPU sample index */
+extern	int		oldCpuIndex;		/* old CPU sample index */
+extern	BOOL		ancientFlag;		/* seeing pre-existing procs */
+extern	BOOL		showThreads;		/* show threads */
+extern	BOOL		noCopy;			/* don't copy data for threads */
+extern	BOOL		noSelf;			/* don't show myself */
+extern	BOOL		noRoot;			/* don't show root procs */
+extern	BOOL		noHeader;		/* don't show column header */
+extern	BOOL		isInfoShown;		/* show info line at top */
+extern	BOOL		myProcs;		/* only show my procs */
+extern	BOOL		activeOnly;		/* only show active procs */
+extern	BOOL		clearScreen;		/* clear screen each loop */
+extern	BOOL		isLooping;		/* loop showing status */
+extern	BOOL		isRunning;		/* we still want to run */
+extern	BOOL		isFrozen;		/* data collection is frozen */
+extern	BOOL		isUpdateForced;		/* update once even if frozen */
+extern	BOOL		isRefreshNeeded;	/* need to refresh display */
+extern	BOOL		isVertical;		/* vertical output format */
+extern	BOOL		isTopMode;		/* top option was used */
+extern	BOOL		isTopAuto;		/* autosize height for top */
+extern	BOOL		useOpenFiles;		/* using open file info */
+extern	BOOL		useCurrentDirectory;	/* using current dir info */
+extern	BOOL		useRootDirectory;	/* using root dir info */
+extern	BOOL		useExecInode;		/* using executable info */
+extern	BOOL		useDeviceNames;		/* using device name info */
+extern	BOOL		useUserNames;		/* using user name info */
+extern	BOOL		useGroupNames;		/* using group name info */
+extern	BOOL		useInitSleep;		/* using initial sleep */
+extern	BOOL		useCommand;		/* using command line info */
+extern	BOOL		useSelf;		/* using my own proc info */
+extern	BOOL		useEnvironment;		/* using environment info */
+extern	BOOL		useWaitChan;		/* using wait channel symbol */
+extern	BOOL		useThreads;		/* use thread data */
+extern	BOOL		useStdioTable[3];	/* using various stdio info */
+extern	pid_t		myPid;			/* my pid */
+extern	uid_t		myUid;			/* my real user id */
+extern	gid_t		myGid;			/* my real group id */
+extern	dev_t		nullDevice;		/* device of /dev/null */
+extern	ino_t		nullInode;		/* inode of /dev/null */
+extern	int		procAllocCount;		/* allocated proc structures */
 extern	int		deathTime;		/* seconds for dead processes */
-extern	int		active_time;		/* seconds for active procs */
-extern	int		pid_count;		/* pids in pid_list */
-extern	int		user_count;		/* users in user_list */
-extern	int		group_count;		/* groups in group_list */
-extern	int		program_count;		/* programs in program_list */
-extern	int		width;			/* width of output */
-extern	int		height;			/* height of output */
+extern	int		activeTime;		/* seconds for active procs */
+extern	int		pidCount;		/* pids in pidList */
+extern	int		userCount;		/* users in userList */
+extern	int		groupCount;		/* groups in groupList */
+extern	int		programCount;		/* programs in programList */
+extern	int		outputWidth;		/* width of output */
+extern	int		outputHeight;		/* height of output */
 extern	int		separation;		/* blanks between columns */
-extern	int		sleep_time_ms;		/* milliseconds between loops */
-extern	int		sync_time;		/* seconds between syncs */
-extern	int		init_time;		/* seconds for initial sleep */
-extern	int		top_count;		/* number of procs for top */
+extern	int		sleepTimeMs;		/* milliseconds between loops */
+extern	int		syncTime;		/* seconds between syncs */
+extern	int		initSleepTime;		/* seconds for initial sleep */
+extern	int		topCount;		/* number of procs for top */
+extern	int		percentSeconds;		/* seconds for cpu percentages */
 extern	int		scrollSeconds;		/* seconds between scrolling */
 extern	int		overlapLines;		/* lines of overlap */
 extern	int		skipCount;		/* lines to skip in display */
 extern	int		procShowCount;		/* processes wanting showing */
 extern	int		procTotalCount;		/* count of all processes */
-extern	PROC *		process_list;		/* list of existent procs */
-extern	PROC *		free_process_list;	/* free proc structure list */
+extern	int		threadShowCount;	/* threads wanting showing */
+extern	int		threadTotalCount;	/* count of all threads */
+extern	int		infoColorId;		/* color id for info line */
+extern	int		headerColorId;		/* color id for header line */
+extern	PROC *		processList;		/* list of existent procs */
+extern	PROC *		freeProcessList;	/* free proc structure list */
 extern	char *		geometry;		/* window geometry string */
 extern	char *		fontName;		/* font name */
 extern	char *		foregroundName;		/* foreground color name */
 extern	char *		backgroundName;		/* background color name */
 extern	char *		displayName;		/* display name */
 extern	const char *	displayType;		/* display type */
-extern	char		empty_string[4];	/* empty string */
-extern	char		root_string[4];		/* root path string */
-extern	pid_t		pid_list[MAX_PIDS];	/* pids to be shown */
-extern	uid_t		user_list[MAX_USERS];	/* user ids to be shown */
-extern	gid_t		group_list[MAX_GROUPS];	/* group ids to be shown */
-extern	char		program_list[MAX_PROGRAMS][MAX_PROGRAM_LEN + 2];
+extern	char		emptyString[4];		/* empty string */
+extern	char		rootString[4];		/* root path string */
+extern	pid_t		pidList[MAX_PIDS];	/* pids to be shown */
+extern	uid_t		userList[MAX_USERS];	/* user ids to be shown */
+extern	gid_t		groupList[MAX_GROUPS];	/* group ids to be shown */
+extern	char		programList[MAX_PROGRAMS][MAX_PROGRAM_LEN + 2];
 
 
 /*
@@ -487,10 +545,10 @@ extern	BOOL	ParseUserInitFile(void);
 extern	BOOL	ParseFile(const char * name, BOOL isOptional);
 extern	BOOL	ParseOptions(ARGS * ap, int depth);
 extern	BOOL	ExpandOptionName(const char * name, int depth);
-extern	long	GetDecimalNumber(char ** cpp);
-extern	double	GetFloatingNumber(char ** cpp);
+extern	long	GetDecimalNumber(const char ** cpp);
+extern	double	GetFloatingNumber(const char ** cpp);
 extern	void	ClearCondition(void);
-extern	BOOL	ParseCondition(char * str);
+extern	BOOL	ParseCondition(const char * str);
 extern	USEFLAG	GetConditionUseFlags(void);
 extern	USEFLAG	GetSortingUseFlags(void);
 extern	void	ClearSorting(void);
@@ -498,7 +556,7 @@ extern	void	SortProcesses(void);
 extern	void	UpdateProcessCounts(void);
 extern	BOOL	AppendColumnSort(ARGS * ap, BOOL reverse);
 extern	BOOL	AppendExpressionSort(ARGS * ap, BOOL reverse);
-extern	PROC *	FindProcess(pid_t pid);
+extern	PROC *	FindProcess(pid_t pid, pthread_t tid);
 extern	BOOL	InitializeProcessData(void);
 extern	BOOL	InitializeDisplay(void);
 extern	void	TopPage(void);
@@ -518,13 +576,17 @@ extern	void	CollectUserNames(void);
 extern	void	CollectGroupNames(void);
 extern	void	CollectDeviceNames(void);
 extern	void	CollectStaticSystemInfo(void);
+extern	void	CollectDynamicSystemInfo(void);
 extern	uid_t	FindUserId(const char * name);
 extern	gid_t	FindGroupId(const char * name);
 extern	const char *	FindUserName(uid_t uid);
 extern	const char *	FindGroupName(gid_t gid);
 extern	const char *	FindDeviceName(dev_t devid);
 extern	const char *	FindDeviceFromInode(dev_t dev, ino_t inode);
+extern	void	CalculateCpuPercentage(PROC * proc);
 extern	void	UpdateTimes(void);
+extern	void	SetCommandLine(PROC * proc, const char * str, int len);
+extern	BOOL	SetSharedString(char ** saveStr, int * saveLen, const char * str, int len);
 extern	void	ResetScrollTime(void);
 extern	void	WaitForCommands(long milliSeconds);
 extern	BOOL	ReadCommands(void);
@@ -541,9 +603,31 @@ extern	long	ElapsedMilliSeconds(const struct timeval * oldTime,
 extern	BOOL	CompareValues(const VALUE leftval,
 			const VALUE rightval, int * result);
 
+extern	ULONG	GetRunOrder(const PROC * proc);
+extern	time_t	GetLastActiveTime(const PROC * proc);
+extern	BOOL	GetIsActive(const PROC * proc);
+extern	int	GetState(const PROC * proc);
+extern	void	BuildStates(PROC * proc);
+extern	int	PickBestState(int state1, int state2);
+
+extern	void	InitializeColors(void);
+extern	int	AllocateColor(const char * color);
+extern	BOOL	DefineColors(void);
+
+extern	void	ClearRowColorConditions(void);
+extern	BOOL	ParseRowColorCondition(const char * color, const char * condition);
+extern	int	EvaluateRowColor(const PROC * proc);
+extern	USEFLAG	GetRowColorUseFlags(void);
+
 extern	BOOL	DpySetDisplay(const char * type);
 extern	BOOL	DpyOpen(void);
+
+extern	BOOL	DpyDefineColor(int colorId,
+	const char * foreground, const char * background, int flags);
+
+extern	void	DpyCreateWindow(void);
 extern	void	DpyClose(void);
+extern	void	DpySetColor(int colorId);
 extern	void	DpyRefresh(void);
 extern	void	DpyBeginPage(void);
 extern	void	DpyChar(int ch);
