@@ -2,7 +2,7 @@
  * Configurable ps-like program.
  * Linux-specific process information collection routines.
  *
- * Copyright (c) 2008 David I. Bell
+ * Copyright (c) 2010 David I. Bell
  * Permission is granted to use, distribute, or modify this source,
  * provided that this copyright notice remains intact.
  */
@@ -29,13 +29,13 @@
 /*
  * Static variables.
  */
-static	DIR *	procdir;		/* opendir for /proc */
+static	DIR *	procDir;		/* opendir for /proc */
 
 
 /*
  * Local procedures.
  */
-static	void	ExamineProcessId(int pid);
+static	void	ExamineProcessId(pid_t pid, pthread_t tid);
 static	void	GetProcessCommandLine(PROC * proc);
 static	void	GetProcessEnvironment(PROC * proc);
 static	void	GetProcessOpenFileCount(PROC * proc);
@@ -44,6 +44,8 @@ static	void	GetProcessRootDirectory(PROC * proc);
 static	void	GetProcessExecInode(PROC * proc);
 static	void	GetProcessStdioDescriptors(PROC * proc);
 static	void	GetProcessWaitSymbol(PROC * proc);
+static	void	ScanThreads(PROC * proc);
+static	BOOL	IsCopyAllowed(const PROC * proc);
 
 static	BOOL	ReadLinkPath(const char * name, char ** retpath,
 			int * retpathlength);
@@ -58,28 +60,28 @@ InitializeProcessData(void)
 {
 	CollectStaticSystemInfo();
 
-	if (use_user_names)
+	if (useUserNames)
 		CollectUserNames();
 
-	if (use_group_names)
+	if (useGroupNames)
 		CollectGroupNames();
 
-	if (use_device_names)
+	if (useDeviceNames)
 		CollectDeviceNames();
 
 	/*
 	 * Open the /proc directory so that the names in it can be found.
 	 */
-	procdir = opendir(PROCDIR);
+	procDir = opendir(PROCDIR);
 
-	if (procdir == NULL)
+	if (procDir == NULL)
 	{
 		fprintf(stderr, "Cannot open %s\n", PROCDIR);
 
 		return FALSE;
 	}
 
-	ancient_flag = TRUE;
+	ancientFlag = TRUE;
 
 	return TRUE;
 }
@@ -92,11 +94,27 @@ InitializeProcessData(void)
 void
 CollectStaticSystemInfo(void)
 {
-	char *	cp;
+	const char *	cp;
 	int	fd;
 	int	cc;
 	char	buf[256];
 
+	/*
+	 * Get the number of ticks per second.
+	 */
+	ticksPerSecond = sysconf(_SC_CLK_TCK);
+
+	if (ticksPerSecond <= 0)
+		ticksPerSecond = 100;
+
+	/*
+	 * Get the page size.
+	 */
+	pageSize = sysconf(_SC_PAGESIZE);
+
+	if (pageSize <= 0)
+		pageSize = 4096;
+
 	/*
 	 * Collect the amount of memory on the system.
 	 */
@@ -121,14 +139,79 @@ CollectStaticSystemInfo(void)
 
 	cp += 9;
 
-	total_memory_clicks = GetDecimalNumber(&cp) / (CLICK_SIZE / 1024);
+	totalMemoryClicks = GetDecimalNumber(&cp) / (pageSize / 1024);
 
 	/*
 	 * Get the starting uptime and time of day.
 	 * This will be used to determine the age of processes.
 	 */
-	start_uptime = GetUptime();
-	start_time = time(NULL);
+	startUptime = GetUptime();
+	startTime = time(NULL);
+}
+
+
+/*
+ * Collect system information that we need.
+ * This information collected here is dynamic.
+ * For now all we need is the process and thread counts.
+ */
+void
+CollectDynamicSystemInfo(void)
+{
+	const struct dirent *	dp;
+	const char *	cp;
+	int	fd;
+	int	cc;
+	char	buf[256];
+
+	/*
+	 * Read the load average information.
+	 * This has a format like: 0.03 0.06 0.07 1/214 12496
+	 */
+	fd = open(PROCDIR "/loadavg", O_RDONLY);
+
+	if (fd < 0)
+		return;
+
+	cc = read(fd, buf, sizeof(buf) - 1);
+
+	(void) close(fd);
+
+	if (cc <= 0)
+		return;
+
+	buf[cc] = '\0';
+
+	/*
+	 * The number of threads on the system follows a slash.
+	 */
+	cp = strchr(buf, '/');
+
+	if (cp == 0)
+		return;
+
+	cp++;
+
+	threadTotalCount = GetDecimalNumber(&cp);
+
+	/*
+	 * Get the number of processes by counting the number of
+	 * filenames in the /proc directory which are numeric.
+	 */
+	procTotalCount = 0;
+
+	seekdir(procDir, 0);
+
+	while ((dp = readdir(procDir)) != NULL)
+	{
+		cp = dp->d_name;
+
+		while (isDigit(*cp))
+			cp++;
+
+		if (*cp == '\0')
+			procTotalCount++;
+	}
 }
 
 
@@ -140,12 +223,12 @@ CollectStaticSystemInfo(void)
 ULONG
 GetUptime(void)
 {
-	char *	cp;
+	const char *	cp;
 	int	fd;
 	int	cc;
-	ULONG	intval;
-	ULONG	fracval;
-	ULONG	fracscale;
+	ULONG	intVal;
+	ULONG	fracVal;
+	ULONG	fracScale;
 	char	buf[128];
 
 	fd = open(PROCDIR "/uptime", O_RDONLY);
@@ -164,29 +247,29 @@ GetUptime(void)
 
 	cp = buf;
 
-	while (isblank(*cp))
+	while (isBlank(*cp))
 		cp++;
 
-	intval = 0;
-	fracval = 0;
-	fracscale = 1;
+	intVal = 0;
+	fracVal = 0;
+	fracScale = 1;
 
-	while (isdigit(*cp))
-		intval = intval * 10 + *cp++ - '0';
+	while (isDigit(*cp))
+		intVal = intVal * 10 + *cp++ - '0';
 
 	if (*cp == '.')
 		cp++;
 
-	while (isdigit(*cp))
+	while (isDigit(*cp))
 	{
-		fracval = fracval * 10 + *cp++ - '0';
-		fracscale = fracscale * 10;
+		fracVal = fracVal * 10 + *cp++ - '0';
+		fracScale = fracScale * 10;
 	}
 
 	if ((*cp != ' ') && (*cp != '\n'))
 		return 0;
 
-	return (intval * TICKS) + ((fracval * TICKS) / fracscale);
+	return (intVal * ticksPerSecond) + ((fracVal * ticksPerSecond) / fracScale);
 }
 
 
@@ -198,48 +281,50 @@ ScanProcesses(void)
 {
 	const struct dirent *	dp;
 	const char *		name;
-	int			pid;
+	pid_t			pid;
 	int			i;
 
 	UpdateTimes();
 
+	CollectDynamicSystemInfo();
+
 	/*
 	 * If we require our own process information, then get that now.
 	 */
-	if (use_self)
-		ExamineProcessId(my_pid);
+	if (useSelf)
+		ExamineProcessId(myPid, NO_THREAD_ID);
 
 	/*
 	 * If there were no specific pids given, then scan them all by
 	 * reading the numeric entries from the "/proc" directory.
 	 * Otherwise, examine just the specified processes.
 	 */
-	if (pid_count == 0)
+	if (pidCount == 0)
 	{
-		seekdir(procdir, 0);
+		seekdir(procDir, 0);
 
-		while ((dp = readdir(procdir)) != NULL)
+		while ((dp = readdir(procDir)) != NULL)
 		{
 			name = dp->d_name;
 
 			pid = 0;
 
-			while (isdigit(*name))
+			while (isDigit(*name))
 				pid = pid * 10 + *name++ - '0';
 
 			if (*name)
 				continue;
 
-			if ((pid != my_pid) || !use_self)
-				ExamineProcessId(pid);
+			if ((pid != myPid) || !useSelf)
+				ExamineProcessId(pid, NO_THREAD_ID);
 		}
 	}
 	else
 	{
-		for (i = 0; i < pid_count; i++)
+		for (i = 0; i < pidCount; i++)
 		{
-			if ((pid_list[i] != my_pid) || !use_self)
-				ExamineProcessId(pid_list[i]);
+			if ((pidList[i] != myPid) || !useSelf)
+				ExamineProcessId(pidList[i], NO_THREAD_ID);
 		}
 	}
 
@@ -249,43 +334,48 @@ ScanProcesses(void)
 
 	UpdateProcessCounts();
 
-	ancient_flag = FALSE;
+	ancientFlag = FALSE;
 }
 
 
 /*
- * Collect data about the specified process id.
+ * Collect data about the specified process and thread id.
+ * A thread id of NO_THREAD_ID collects the main process data,
+ * possibly along with the process data of all of its threads.
  * This allocates a new PROC structure if necessary.
  * If the process is successfully examined, the valid flag is set.
  */
 static void
-ExamineProcessId(int pid)
+ExamineProcessId(pid_t pid, pthread_t tid)
 {
 	PROC *		proc;
 	int		fd;
 	int		cc;
 	int		i;
-	char *		begname;
-	char *		endname;
-	char *		cp;
-	long		ticks_from_start;
-	long		seconds_from_start;
-	BOOL		okskip;
-	struct	stat	statbuf;
+	char *		begName;
+	char *		endName;
+	const char *	cp;
+	long		ticksFromStart;
+	long		secondsFromStart;
+	BOOL		okSkip;
+	struct	stat	statBuf;
 	char		buf[512];
 	char		name[PROC_FILE_LEN];
 
-	proc = FindProcess(pid);
+	proc = FindProcess(pid, tid);
 
-	proc->isvalid = FALSE;
-	proc->isshown = FALSE;
+	proc->isValid = FALSE;
+	proc->isShown = FALSE;
 
-	okskip = ((pid != my_pid) || !use_self);
+	okSkip = ((pid != myPid) || !useSelf);
 
-	if (okskip && no_self && (pid == my_pid))
+	if (okSkip && noSelf && (pid == myPid))
 		return;
 
-	sprintf(name, "%s/%d/stat", PROCDIR, proc->pid);
+	if (proc->isThread)
+		sprintf(name, "%s/%ld/task/%ld/stat", PROCDIR, (long) pid, (long) tid);
+	else
+		sprintf(name, "%s/%ld/stat", PROCDIR, (long) pid);
 
 	fd = open(name, O_RDONLY);
 
@@ -297,7 +387,7 @@ ExamineProcessId(int pid)
 		return;
 	}
 
-	if (fstat(fd, &statbuf) < 0)
+	if (fstat(fd, &statBuf) < 0)
 	{
 		(void) close(fd);
 
@@ -306,27 +396,27 @@ ExamineProcessId(int pid)
 
 	proc->deathTime = 0;
 
-	if (okskip)
+	if (okSkip)
 	{
-		if (my_procs && (statbuf.st_uid != my_uid))
+		if (myProcs && (statBuf.st_uid != myUid))
 		{
 			(void) close(fd);
 
 			return;
 		}
 
-		if (no_root && (statbuf.st_uid == 0))
+		if (noRoot && (statBuf.st_uid == 0))
 			return;
 
-		if ((user_count > 0))
+		if ((userCount > 0))
 		{
-			for (i = 0; i < user_count; i++)
+			for (i = 0; i < userCount; i++)
 			{
-				if (statbuf.st_uid == user_list[i])
+				if (statBuf.st_uid == userList[i])
 					break;
 			}
 
-			if (i == user_count)
+			if (i == userCount)
 			{
 				(void) close(fd);
 
@@ -334,15 +424,15 @@ ExamineProcessId(int pid)
 			}
 		}
 
-		if ((group_count > 0))
+		if ((groupCount > 0))
 		{
-			for (i = 0; i < group_count; i++)
+			for (i = 0; i < groupCount; i++)
 			{
-				if (statbuf.st_gid == group_list[i])
+				if (statBuf.st_gid == groupList[i])
 					break;
 			}
 
-			if (i == group_count)
+			if (i == groupCount)
 			{
 				(void) close(fd);
 
@@ -351,8 +441,8 @@ ExamineProcessId(int pid)
 		}
 	}
 
-	proc->uid = statbuf.st_uid;
-	proc->gid = statbuf.st_gid;
+	proc->uid = statBuf.st_uid;
+	proc->gid = statBuf.st_gid;
 
 	/*
 	 * Read the process status into a buffer.
@@ -381,39 +471,39 @@ ExamineProcessId(int pid)
 	 * The program name begins after a left parenthesis.
 	 * Break the status string into two at that point.
 	 */
-	begname = strchr(buf, BEGNAMECHAR);
+	begName = strchr(buf, BEGNAMECHAR);
 
-	if (begname == NULL)
+	if (begName == NULL)
 	{
 		fprintf(stderr, "Cannot find start of program name\n");
 
 		return;
 	}
 
-	*begname++ = '\0';
+	*begName++ = '\0';
 
 	/*
 	 * The program name ends after a right parenthesis.
 	 * But, look for the rightmost one in case the program name
 	 * itself contains a parenthesis!
 	 */
-	endname = strchr(begname, ENDNAMECHAR);
+	endName = strchr(begName, ENDNAMECHAR);
 
-	if (endname == NULL)
+	if (endName == NULL)
 	{
 		fprintf(stderr, "Cannot find end of program name\n");
 
 		return;
 	}
 
-	while ((cp = strchr(endname + 1, ENDNAMECHAR)) != NULL)
-		endname = cp;
+	while ((cp = strchr(endName + 1, ENDNAMECHAR)) != NULL)
+		endName = begName + (cp - begName);
 
-	MakePrintable(begname, endname - begname);
+	MakePrintable(begName, endName - begName);
 
-	*endname++ = '\0';
+	*endName++ = '\0';
 
-	strncpy(proc->program, begname, MAX_PROGRAM_LEN);
+	strncpy(proc->program, begName, MAX_PROGRAM_LEN);
 
 	proc->program[MAX_PROGRAM_LEN] = '\0';
 
@@ -421,12 +511,12 @@ ExamineProcessId(int pid)
 	 * Find the process state character, and then parse the numbers on
 	 * the rest of the line to collect the remaining state information.
 	 */
-	cp = endname;
+	cp = endName;
 
-	while (isblank(*cp))
+	while (isBlank(*cp))
 		cp++;
 
-	if ((*cp == '\0') || (*cp == '\n') || isdigit(*cp))
+	if ((*cp == '\0') || (*cp == '\n') || isDigit(*cp))
 	{
 		fprintf(stderr, "Bad proc state character\n");
 
@@ -434,42 +524,43 @@ ExamineProcessId(int pid)
 	}
 
 	proc->state = *cp++;
-
-	proc->parent_pid = GetDecimalNumber(&cp);
-	proc->process_group = GetDecimalNumber(&cp);
-	proc->session_id = GetDecimalNumber(&cp);
-	proc->tty_dev = GetDecimalNumber(&cp);
-	proc->tty_pgrp = GetDecimalNumber(&cp);
+	proc->states[0] = proc->state;
+	proc->states[1] = '\0';
+	proc->parentPid = GetDecimalNumber(&cp);
+	proc->processGroup = GetDecimalNumber(&cp);
+	proc->sessionId = GetDecimalNumber(&cp);
+	proc->ttyDevice = GetDecimalNumber(&cp);
+	proc->ttyProcessGroup = GetDecimalNumber(&cp);
 	proc->flags = GetDecimalNumber(&cp);
-	proc->min_flt = GetDecimalNumber(&cp);
-	proc->cmin_flt = GetDecimalNumber(&cp);
-	proc->maj_flt = GetDecimalNumber(&cp);
-	proc->cmaj_flt = GetDecimalNumber(&cp);
-	proc->utime = GetDecimalNumber(&cp);
-	proc->stime = GetDecimalNumber(&cp);
-	proc->cutime = GetDecimalNumber(&cp);
-	proc->cstime = GetDecimalNumber(&cp);
+	proc->minorFaults = GetDecimalNumber(&cp);
+	proc->childMinorFaults = GetDecimalNumber(&cp);
+	proc->majorFaults = GetDecimalNumber(&cp);
+	proc->childMajorFaults = GetDecimalNumber(&cp);
+	proc->userRunTime = GetDecimalNumber(&cp);
+	proc->systemRunTime = GetDecimalNumber(&cp);
+	proc->childUserRunTime = GetDecimalNumber(&cp);
+	proc->childSystemRunTime = GetDecimalNumber(&cp);
 	proc->priority = GetDecimalNumber(&cp);
 	proc->nice = GetDecimalNumber(&cp);
 	proc->threadCount = GetDecimalNumber(&cp);
-	proc->it_real_value = GetDecimalNumber(&cp);
-	proc->start_time_ticks = GetDecimalNumber(&cp);
-	proc->vsize = GetDecimalNumber(&cp);
+	proc->itRealValue = GetDecimalNumber(&cp);
+	proc->startTimeTicks = GetDecimalNumber(&cp);
+	proc->virtualSize = GetDecimalNumber(&cp);
 	proc->rss = GetDecimalNumber(&cp);
-	proc->rss_limit = GetDecimalNumber(&cp);
-	proc->start_code = GetDecimalNumber(&cp);
-	proc->end_code = GetDecimalNumber(&cp);
-	proc->start_stack = GetDecimalNumber(&cp);
+	proc->rssLimit = GetDecimalNumber(&cp);
+	proc->startCode = GetDecimalNumber(&cp);
+	proc->endCode = GetDecimalNumber(&cp);
+	proc->startStack = GetDecimalNumber(&cp);
 	proc->esp = GetDecimalNumber(&cp);
 	proc->eip = GetDecimalNumber(&cp);
 	proc->signal = GetDecimalNumber(&cp);
-	proc->sigblock = GetDecimalNumber(&cp);
-	proc->sigignore = GetDecimalNumber(&cp);
-	proc->sigcatch = GetDecimalNumber(&cp);
-	proc->wchan = GetDecimalNumber(&cp);
-	proc->nswap = GetDecimalNumber(&cp);
-	proc->cnswap = GetDecimalNumber(&cp);
-	proc->exitsignal = GetDecimalNumber(&cp);
+	proc->sigBlock = GetDecimalNumber(&cp);
+	proc->sigIgnore = GetDecimalNumber(&cp);
+	proc->sigCatch = GetDecimalNumber(&cp);
+	proc->waitChan = GetDecimalNumber(&cp);
+	proc->pagesSwapped = GetDecimalNumber(&cp);
+	proc->childPagesSwapped = GetDecimalNumber(&cp);
+	proc->exitSignal = GetDecimalNumber(&cp);
 	proc->processor = GetDecimalNumber(&cp);
 	proc->realTimePriority = GetDecimalNumber(&cp);
 	proc->policy = GetDecimalNumber(&cp);
@@ -481,18 +572,25 @@ ExamineProcessId(int pid)
 	 * This can be positive or negative according to whether the
 	 * process started before or after us.
 	 */
-	ticks_from_start = proc->start_time_ticks - start_uptime;
+	ticksFromStart = proc->startTimeTicks - startUptime;
 
-	if (ticks_from_start >= 0)
-		seconds_from_start = ticks_from_start / TICKS;
+	if (ticksFromStart >= 0)
+		secondsFromStart = ticksFromStart / ticksPerSecond;
 	else
-		seconds_from_start = -((-ticks_from_start) / TICKS);
+		secondsFromStart = -((-ticksFromStart) / ticksPerSecond);
 
 	/*
 	 * Add the elapsed seconds to the clock time when we started
 	 * to get the clock time the process started.
 	 */
-	proc->start_time_clock = start_time + seconds_from_start;
+	proc->startTimeClock = startTime + secondsFromStart;
+
+	/*
+	 * If the process is newly seen then save its starting runtime.
+	 * This is used for the cpu percentage calculation.
+	 */
+	if (proc->isNew)
+		proc->firstCpuTime = proc->userRunTime + proc->systemRunTime;
 
 	/*
 	 * See if the process has changed state, and is therefore active.
@@ -505,10 +603,10 @@ ExamineProcessId(int pid)
 	 * However, get the extra data always if it is older than
 	 * the specified sync time.
 	 */
-	if (proc->changed ||
-		((proc->last_sync_time + sync_time) <= current_time))
+	if (proc->isChanged ||
+		((proc->lastSyncTime + syncTime) <= currentTime))
 	{
-		proc->last_sync_time = current_time;
+		proc->lastSyncTime = currentTime;
 
 		GetProcessOpenFileCount(proc);
 		GetProcessStdioDescriptors(proc);
@@ -520,11 +618,70 @@ ExamineProcessId(int pid)
 		GetProcessWaitSymbol(proc);
 	}
 
-	proc->live_counter = live_counter;
-	proc->isvalid = TRUE;
+	proc->liveCounter = liveCounter;
+	proc->isValid = TRUE;
 
 	if (IsShownProcess(proc))
-		proc->isshown = TRUE;
+		proc->isShown = TRUE;
+
+	/*
+	 * If we are showing or using threads and this process has more than
+	 * one thread then collect information on them.
+	 */
+	if (showThreads || useThreads)
+		ScanThreads(proc);
+
+	/*
+	 * Store the states string for threads if any.
+	 */
+	BuildStates(proc);
+}
+
+
+/*
+ * Scan the list of threads for the indicated process.
+ */
+static void
+ScanThreads(PROC * proc)
+{
+	DIR *			dir;
+	const struct dirent *	dp;
+	const char *		cp;
+	pthread_t		tid;
+	char			name[PROC_FILE_LEN];
+
+	/*
+	 * If this is a thread or there aren't multiple threads then
+	 * do nothing.
+	 */
+	if (proc->isThread || (proc->threadCount <= 1))
+		return;
+
+	/*
+	 * Read the list of thread ids from the task directory of the
+	 * main process and examine each one.
+	 */
+	sprintf(name, "%s/%ld/task", PROCDIR, (long) proc->pid);
+
+	dir = opendir(name);
+
+	if (dir == NULL)
+		return;
+
+	while ((dp = readdir(dir)) != NULL)
+	{
+		cp = dp->d_name;
+
+		tid = 0;
+
+		while (isDigit(*cp))
+			tid = tid * 10 + (*cp++ - '0');
+
+		if (*cp == '\0')
+			ExamineProcessId(proc->pid, tid);
+	}
+
+	closedir(dir);
 }
 
 
@@ -542,9 +699,9 @@ GetProcessWaitSymbol(PROC * proc)
 	 * If the wait channel is 0 or is not used then
 	 * set an empty symbol name.
 	 */
-	if (!use_wchan || (proc->wchan == 0))
+	if (!useWaitChan || (proc->waitChan == 0))
 	{
-		proc->wchan_symbol[0] = '\0';
+		proc->waitChanSymbol[0] = '\0';
 
 		return;
 	}
@@ -553,14 +710,14 @@ GetProcessWaitSymbol(PROC * proc)
 	 * Open the wait channel file and read it into a buffer
 	 * including trying to read one extra character.
 	 */
-	sprintf(name, "%s/%d/wchan", PROCDIR, proc->pid);
+	sprintf(name, "%s/%ld/waitChan", PROCDIR, (long) proc->pid);
 
 	fd = open(name, O_RDONLY);
 	len = -1;
 
 	if (fd >= 0)
 	{
-		len = read(fd, proc->wchan_symbol, MAX_WCHAN_LEN + 1);
+		len = read(fd, proc->waitChanSymbol, MAX_WCHAN_LEN + 1);
 
 		(void) close(fd);
 	}
@@ -570,8 +727,8 @@ GetProcessWaitSymbol(PROC * proc)
 	 */
 	if (len < 0)
 	{
-		proc->wchan_symbol[0] = '-';
-		proc->wchan_symbol[1] = '\0';
+		proc->waitChanSymbol[0] = '-';
+		proc->waitChanSymbol[1] = '\0';
 
 		return;
 	}
@@ -584,15 +741,15 @@ GetProcessWaitSymbol(PROC * proc)
 	if (len > MAX_WCHAN_LEN)
 	{
 		len = MAX_WCHAN_LEN;
-		proc->wchan_symbol[MAX_WCHAN_LEN - 1] = '|';
+		proc->waitChanSymbol[MAX_WCHAN_LEN - 1] = '|';
 	}
 
 	/*
 	 * Null terminate the wait symbol name and make it printable.
 	 */
-	proc->wchan_symbol[len] = '\0';
+	proc->waitChanSymbol[len] = '\0';
 
-	MakePrintable(proc->wchan_symbol, len);
+	MakePrintable(proc->waitChanSymbol, len);
 }
 
 
@@ -601,6 +758,7 @@ GetProcessWaitSymbol(PROC * proc)
  * If there isn't one, then use the program name as the command line,
  * surrounded by parenthesis.  If the command line is small, then it
  * fits within the proc structure, otherwise we have to malloc it.
+ * Threads copy the command line string from the main process.
  */
 static void
 GetProcessCommandLine(PROC * proc)
@@ -612,22 +770,33 @@ GetProcessCommandLine(PROC * proc)
 
 	len = 0;
 
-	if (!use_command)
+	if (!useCommand)
 	{
-		proc->hascommand = FALSE;
-		proc->command_length = 0;
+		proc->hasCommand = FALSE;
+		proc->commandLength = 0;
 		proc->command[0] = '\0';
 
 		return;
 	}
 
-	proc->hascommand = TRUE;
+	proc->hasCommand = TRUE;
+
+	/*
+	 * If we are a thread process and have an owner then copy the
+	 * command from the owner structure.
+	 */
+	if (IsCopyAllowed(proc))
+	{
+		SetCommandLine(proc, proc->owner->command, proc->owner->commandLength);
+
+		return;
+	}
 
 	/*
 	 * Open the command line file and read it into a large buffer,
 	 * including trying to read one extra character.
 	 */
-	sprintf(name, "%s/%d/cmdline", PROCDIR, proc->pid);
+	sprintf(name, "%s/%ld/cmdline", PROCDIR, (long) proc->pid);
 
 	fd = open(name, O_RDONLY);
 
@@ -645,7 +814,7 @@ GetProcessCommandLine(PROC * proc)
 	 */
 	if ((fd < 0) || (len <= 0))
 	{
-		proc->hascommand = FALSE;
+		proc->hasCommand = FALSE;
 		len = strlen(proc->program);
 
 		buffer[0] = '(';
@@ -674,54 +843,17 @@ GetProcessCommandLine(PROC * proc)
 	MakePrintable(buffer, len);
 
 	/*
-	 * See if the command line is the same as last time.
-	 * If so, then we are done.
+	 * Store the command line into the structure.
 	 */
-	if ((len == proc->command_length) &&
-		(buffer[0] == proc->command[0]) &&
-		(buffer[len - 1] == proc->command[len - 1]) &&
-		(memcmp(buffer, proc->command, len) == 0))
-	{
-		return;
-	}
-
-	/*
-	 * Free any old command line buffer that was there, and point
-	 * back to the space already allocated in the proc structure.
-	 */
-	if (proc->command != proc->command_buffer)
-		free(proc->command);
-
-	proc->command = proc->command_buffer;
-
-	/*
-	 * If the command line is too large for the proc buffer,
-	 * then allocate a new one.
-	 */
-	if (len > BUF_COMMAND_LEN)
-	{
-		proc->command = malloc(len + 1);
-
-		if (proc->command == NULL)
-		{
-			fprintf(stderr, "No memory\n");
-
-			exit(1);
-		}
-	}
-
-	/*
-	 * Copy the command line into the command buffer and set its length.
-	 */
-	proc->command_length = len;
-	memcpy(proc->command, buffer, len + 1);
+	SetCommandLine(proc, buffer, len);
 }
 
 
 /*
  * Get the environment for the specified process.
  * This could be very large, so it is allocated dynamically and we
- * attempt to share strings among processes.
+ * attempt to share strings among processes.  Threads copy the
+ * environment string value from the main process.
  */
 static void
 GetProcessEnvironment(PROC * proc)
@@ -731,19 +863,31 @@ GetProcessEnvironment(PROC * proc)
 	char	name[PROC_FILE_LEN];
 	char	buffer[MAX_ENVIRON_LEN + 2];
 
-	len = 0;
-
-	if (!use_environment)
+	if (!useEnvironment)
 		return;
 
+	/*
+	 * If we are a thread process and have an owner then copy the
+	 * environment from the owner structure.
+	 */
+	if (IsCopyAllowed(proc))
+	{
+		SetSharedString(&proc->environment, &proc->environmentLength,
+			proc->owner->environment, proc->owner->environmentLength);
+
+		return;
+	}
+
 	/*
 	 * Open the environment file and read it into a large buffer,
 	 * including trying to read one extra character.
 	 */
-	sprintf(name, "%s/%d/environ", PROCDIR, proc->pid);
+	sprintf(name, "%s/%ld/environ", PROCDIR, (long) proc->pid);
 
 	fd = open(name, O_RDONLY);
 
+	len = 0;
+
 	if (fd >= 0)
 	{
 		len = read(fd, buffer, MAX_ENVIRON_LEN + 1);
@@ -757,11 +901,10 @@ GetProcessEnvironment(PROC * proc)
 	 */
 	if ((fd < 0) || (len <= 0))
 	{
-		if (proc->environment != empty_string)
-			FreeSharedString(proc->environment);
+		FreeSharedString(proc->environment);
 
-		proc->environment = empty_string;
-		proc->environment_length = 0;
+		proc->environment = emptyString;
+		proc->environmentLength = 0;
 
 		return;
 	}
@@ -785,32 +928,10 @@ GetProcessEnvironment(PROC * proc)
 	MakePrintable(buffer, len);
 
 	/*
-	 * Compare the string against what is already there.
-	 * If they are the same, then we don't need to do anything.
+	 * Store the environment line into the structure.
 	 */
-	if ((proc->environment_length == len) &&
-		(proc->environment[0] == buffer[0]) &&
-		(memcmp(proc->environment, buffer, len) == 0))
-	{
-		return;
-	}
-
-	/*
-	 * Free the old string, and copy the buffer into a new shared string.
-	 * If the string is the same as the one from another process (which
-	 * is likely), then it will be shared.
-	 */
-	if (proc->environment != empty_string)
-		FreeSharedString(proc->environment);
-
-	proc->environment = AllocateSharedString(buffer, len);
-	proc->environment_length = len;
-
-	if (proc->environment == NULL)
-	{
-		proc->environment = empty_string;
-		proc->environment_length = 0;
-	}
+	SetSharedString(&proc->environment, &proc->environmentLength,
+		buffer, len);
 }
 
 
@@ -829,12 +950,27 @@ GetProcessOpenFileCount(PROC * proc)
 	int			count;
 	char			name[PROC_FILE_LEN];
 
-	proc->openfiles = -1;
+	proc->openFiles = -1;
 
-	if (!use_open_files)
+	if (!useOpenFiles)
 		return;
 
-	sprintf(name, "%s/%d/fd", PROCDIR, proc->pid);
+	/*
+	 * If we are a thread process and have an owner then copy the
+	 * open file count from the main process.
+	 */
+	if (IsCopyAllowed(proc))
+	{
+		proc->openFiles = proc->owner->openFiles;
+
+		return;
+	}
+
+	/*
+	 * Open the fd directory in the process status and count the
+	 * number of numeric file names.
+	 */
+	sprintf(name, "%s/%ld/fd", PROCDIR, (long) proc->pid);
 
 	dir = opendir(name);
 
@@ -847,7 +983,7 @@ GetProcessOpenFileCount(PROC * proc)
 	{
 		cp = dp->d_name;
 
-		while (isdigit(*cp))
+		while (isDigit(*cp))
 			cp++;
 
 		if (*cp == '\0')
@@ -856,7 +992,7 @@ GetProcessOpenFileCount(PROC * proc)
 
 	closedir(dir);
 
-	proc->openfiles = count;
+	proc->openFiles = count;
 }
 
 
@@ -871,12 +1007,24 @@ GetProcessCurrentDirectory(PROC * proc)
 {
 	char	name[PROC_FILE_LEN];
 
-	if (!use_current_directory)
+	if (!useCurrentDirectory)
 		return;
 
-	sprintf(name, "%s/%d/cwd", PROCDIR, proc->pid);
+	/*
+	 * If we are a thread process and have an owner then copy the
+	 * current directory path from the main process.
+	 */
+	if (IsCopyAllowed(proc))
+	{
+		SetSharedString(&proc->cwdPath, &proc->cwdPathLength,
+			proc->owner->cwdPath, proc->owner->cwdPathLength);
 
-	ReadLinkPath(name, &proc->cwd_path, &proc->cwd_path_length);
+		return;
+	}
+
+	sprintf(name, "%s/%ld/cwd", PROCDIR, (long) proc->pid);
+
+	ReadLinkPath(name, &proc->cwdPath, &proc->cwdPathLength);
 }
 
 
@@ -891,12 +1039,24 @@ GetProcessRootDirectory(PROC * proc)
 {
 	char	name[PROC_FILE_LEN];
 
-	if (!use_root_directory)
+	if (!useRootDirectory)
 		return;
 
-	sprintf(name, "%s/%d/root", PROCDIR, proc->pid);
+	/*
+	 * If we are a thread process and have an owner then copy the
+	 * root directory path from the main process.
+	 */
+	if (IsCopyAllowed(proc))
+	{
+		SetSharedString(&proc->rootPath, &proc->rootPathLength,
+			proc->owner->rootPath, proc->owner->rootPathLength);
 
-	ReadLinkPath(name, &proc->root_path, &proc->root_path_length);
+		return;
+	}
+
+	sprintf(name, "%s/%ld/root", PROCDIR, (long) proc->pid);
+
+	ReadLinkPath(name, &proc->rootPath, &proc->rootPathLength);
 }
 
 
@@ -911,12 +1071,24 @@ GetProcessExecInode(PROC * proc)
 {
 	char	name[PROC_FILE_LEN];
 
-	if (!use_exec_inode)
+	if (!useExecInode)
 		return;
 
-	sprintf(name, "%s/%d/exe", PROCDIR, proc->pid);
+	/*
+	 * If we are a thread process and have an owner then copy the
+	 * executable path from the main process.
+	 */
+	if (IsCopyAllowed(proc))
+	{
+		SetSharedString(&proc->execPath, &proc->execPathLength,
+			proc->owner->execPath, proc->owner->execPathLength);
 
-	ReadLinkPath(name, &proc->exec_path, &proc->exec_path_length);
+		return;
+	}
+
+	sprintf(name, "%s/%ld/exe", PROCDIR, (long) proc->pid);
+
+	ReadLinkPath(name, &proc->execPath, &proc->execPathLength);
 }
 
 
@@ -935,12 +1107,25 @@ GetProcessStdioDescriptors(PROC * proc)
 
 	for (fd = 0; fd <= 2; fd++)
 	{
-		if (!use_stdio[fd])
+		if (!useStdioTable[fd])
 			continue;
 
-		sprintf(name, "%s/%d/fd/%d", PROCDIR, proc->pid, fd);
+		/*
+		 * If we are a thread process and have an owner then copy the
+		 * stdio path from the main process.
+		 */
+		if (IsCopyAllowed(proc))
+		{
+			SetSharedString(&proc->stdioPaths[fd], &dummy,
+				proc->owner->stdioPaths[fd],
+				strlen(proc->owner->stdioPaths[fd]));
 
-		ReadLinkPath(name, &proc->stdio_paths[fd], &dummy);
+			continue;
+		}
+
+		sprintf(name, "%s/%ld/fd/%d", PROCDIR, (long) proc->pid, fd);
+
+		ReadLinkPath(name, &proc->stdioPaths[fd], &dummy);
 	}
 }
 
@@ -955,17 +1140,17 @@ GetProcessStdioDescriptors(PROC * proc)
 static BOOL
 ReadLinkPath(const char * name, char ** retpath, int * retpathlength)
 {
-	char *	newpath;
-	char *	oldpath;
+	char *	newPath;
+	char *	oldPath;
 	int	len;
-	int	oldlength;
+	int	oldLength;
 	char	buffer[MAX_PATH_LEN];
 
 	/*
 	 * Save the values for the old path.
 	 */
-	oldpath = *retpath;
-	oldlength = *retpathlength;
+	oldPath = *retpath;
+	oldLength = *retpathlength;
 
 	/*
 	 * Read the value of the symbolic link.
@@ -974,9 +1159,9 @@ ReadLinkPath(const char * name, char ** retpath, int * retpathlength)
 
 	if ((len <= 0) || (len == sizeof(buffer)))
 	{
-		FreeSharedString(oldpath);
+		FreeSharedString(oldPath);
 
-		*retpath = empty_string;
+		*retpath = emptyString;
 		*retpathlength = 0;
 
 		return FALSE;
@@ -987,32 +1172,67 @@ ReadLinkPath(const char * name, char ** retpath, int * retpathlength)
 	/*
 	 * If the path is the same as the existing one then do nothing.
 	 */
-	if ((len == oldlength) && (strcmp(oldpath, buffer) == 0))
+	if ((len == oldLength) && (strcmp(oldPath, buffer) == 0))
 		return TRUE;
 
 	/*
 	 * The value has changed.  Delete the old string.
 	 */
-	FreeSharedString(oldpath);
+	FreeSharedString(oldPath);
 
-	*retpath = empty_string;
+	*retpath = emptyString;
 	*retpathlength = 0;
 
 	/*
 	 * Allocate a new shared string to store the new value.
 	 */
-	newpath = AllocateSharedString(buffer, len);
+	newPath = AllocateSharedString(buffer, len);
 
-	if (newpath == NULL)
+	if (newPath == NULL)
 		return FALSE;
 
 	/*
 	 * Return the new string value.
 	 */
-	*retpath = newpath;
+	*retpath = newPath;
 	*retpathlength = len;
 
 	return TRUE;
 }
 
+
+/*
+ * Return whether or now we are a thread process that is allowed to
+ * copy data from the owning main process.
+ */
+static BOOL
+IsCopyAllowed(const PROC * proc)
+{
+	if (noCopy)
+		return FALSE;
+
+	return (proc->isThread && (proc->owner != 0));
+}
+
+
+/*
+ * Pick the best state character of the two states which most indicates
+ * what the multiple threads of a process are doing.
+ */
+int
+PickBestState(int state1, int state2)
+{
+	if ((state1 == 'R') || (state2 == 'R'))
+		return 'R';
+
+	if ((state1 == 'D') || (state2 == 'D'))
+		return 'D';
+
+	if ((state1 == 'S') || (state2 == 'S'))
+		return 'S';
+
+	return state1;
+}
+
+
 /* END CODE */
