@@ -2,7 +2,7 @@
  * Configurable ps-like program.
  * Command line option parsing.
  *
- * Copyright (c) 2008 David I. Bell
+ * Copyright (c) 2010 David I. Bell
  * Permission is granted to use, distribute, or modify this source,
  * provided that this copyright notice remains intact.
  */
@@ -13,7 +13,7 @@
 /*
  * Definitions for version printout.
  */
-#define	VERSION		"3.2"
+#define	VERSION		"4.0"
 #define	AUTHOR_NAME	"David I. Bell"
 #define	AUTHOR_EMAIL	"dbell@canb.auug.org.au"
 
@@ -71,7 +71,15 @@
 #define	OPT_INFO		48
 #define	OPT_DEATH_TIME		49
 #define	OPT_GEOMETRY		50
-#define	OPT_LAST_OPTION		50
+#define	OPT_SHOW_THREADS	51
+#define	OPT_USE_THREADS		52
+#define	OPT_NO_COPY		53
+#define	OPT_INFO_COLOR		54
+#define	OPT_HEADER_COLOR	55
+#define	OPT_ROW_COLOR		56
+#define	OPT_PERCENT_SECONDS	57
+
+#define	OPT_LAST_OPTION		57
 
 
 /*
@@ -84,7 +92,7 @@ typedef	struct
 } OPTION;
 
 
-static	OPTION	option_table[] =
+static	OPTION	option_table[OPT_LAST_OPTION + 2] =
 {
 	{"col",		OPT_SET_COLUMNS},
 	{"addcol",	OPT_ADD_COLUMNS},
@@ -137,6 +145,13 @@ static	OPTION	option_table[] =
 	{"overlap",	OPT_OVERLAP_LINES},
 	{"info",	OPT_INFO},
 	{"geometry",	OPT_GEOMETRY},
+	{"showthreads",	OPT_SHOW_THREADS},
+	{"usethreads",	OPT_USE_THREADS},
+	{"nocopy",	OPT_NO_COPY},
+	{"infocolor",	OPT_INFO_COLOR},
+	{"headercolor",	OPT_HEADER_COLOR},
+	{"rowcolor",	OPT_ROW_COLOR},
+	{"percentseconds",	OPT_PERCENT_SECONDS},
 	{NULL,		OPT_ILLEGAL}
 };
 
@@ -167,9 +182,14 @@ static	BOOL	SetBackground(ARGS *);
 static	BOOL	SetDisplay(ARGS *);
 static	BOOL	SetScrollTime(ARGS *);
 static	BOOL	SetOverlapLines(ARGS *);
+static	BOOL	SetInfoColor(ARGS *);
+static	BOOL	SetHeaderColor(ARGS *);
 static	BOOL	SetDefault(ARGS *);
 static	BOOL	AddColumns(ARGS *);
 static	BOOL	RemoveColumns(ARGS *);
+static	BOOL	SetRowColorCondition(ARGS * ap);
+static	BOOL	SetPercentSeconds(ARGS *);
+
 static	void	DefaultOneOption(int);
 static	void	PrintUsage(void);
 static	void	PrintVersion(void);
@@ -186,9 +206,9 @@ ParseOptions(ARGS * ap, int depth)
 	const OPTION *	option;
 	const char *	str;
 	BOOL		status;
-	BOOL		success;
+	BOOL		isSuccess;
 
-	success = TRUE;
+	isSuccess = TRUE;
 
 	while (ap->count-- > 0)
 	{
@@ -274,31 +294,42 @@ ParseOptions(ARGS * ap, int depth)
 				break;
 
 			case OPT_NO_SELF:
-				no_self = TRUE;
+				noSelf = TRUE;
 				break;
 
 			case OPT_NO_ROOT:
-				no_root = TRUE;
+				noRoot = TRUE;
 				break;
 
 			case OPT_NO_HEADER:
-				no_header = TRUE;
+				noHeader = TRUE;
 				break;
 
 			case OPT_INFO:
-				info = TRUE;
+				isInfoShown = TRUE;
 				break;
 
+			case OPT_SHOW_THREADS:
+				showThreads = TRUE;
+				break;
+
+			case OPT_USE_THREADS:
+				useThreads = TRUE;
+				break;
+
+			case OPT_NO_COPY:
+				noCopy = TRUE;
+
 			case OPT_MY_PROCS:
-				my_procs = TRUE;
+				myProcs = TRUE;
 				break;
 
 			case OPT_ACTIVE_ONLY:
-				active_only = TRUE;
+				activeOnly = TRUE;
 				break;
 
 			case OPT_CLEAR_SCREEN:
-				clear_screen = TRUE;
+				clearScreen = TRUE;
 				break;
 
 			case OPT_READ_FILE:
@@ -330,27 +361,27 @@ ParseOptions(ARGS * ap, int depth)
 				break;
 
 			case OPT_ONCE:
-				loop = FALSE;
+				isLooping = FALSE;
 				displayType = DISPLAY_TYPE_TTY;
 				break;
 
 			case OPT_LOOP:
-				loop = TRUE;
+				isLooping = TRUE;
 				displayType = DISPLAY_TYPE_TTY;
 				break;
 
 			case OPT_CURSES:
-				loop = TRUE;
+				isLooping = TRUE;
 				displayType = DISPLAY_TYPE_CURSES;
 				break;
 
 			case OPT_X11:
-				loop = TRUE;
+				isLooping = TRUE;
 				displayType = DISPLAY_TYPE_X11;
 				break;
 
 			case OPT_VERTICAL:
-				vertical = TRUE;
+				isVertical = TRUE;
 				break;
 
 			case OPT_PIDS:
@@ -389,6 +420,10 @@ ParseOptions(ARGS * ap, int depth)
 				status = SetDisplay(ap);
 				break;
 
+			case OPT_PERCENT_SECONDS:
+				status = SetPercentSeconds(ap);
+				break;
+
 			case OPT_SCROLL_TIME:
 				status = SetScrollTime(ap);
 				break;
@@ -397,6 +432,18 @@ ParseOptions(ARGS * ap, int depth)
 				status = SetOverlapLines(ap);
 				break;
 
+			case OPT_INFO_COLOR:
+				status = SetInfoColor(ap);
+				break;
+
+			case OPT_HEADER_COLOR:
+				status = SetHeaderColor(ap);
+				break;
+
+			case OPT_ROW_COLOR:
+				status = SetRowColorCondition(ap);
+				break;
+
 			case OPT_HELP:
 				PrintUsage();
 				break;
@@ -430,10 +477,10 @@ ParseOptions(ARGS * ap, int depth)
 		}
 
 		if (!status)
-			success = FALSE;
+			isSuccess = FALSE;
 	}
 
-	return success;
+	return isSuccess;
 }
 
 
@@ -446,7 +493,7 @@ BOOL
 ExpandOptionName(const char * name, int depth)
 {
 	ARGS	args;
-	char	upname[MAX_MACRO_LEN + 2];
+	char	upName[MAX_MACRO_LEN + 2];
 
 	if (depth > MAX_OPTION_DEPTH)
 	{
@@ -467,16 +514,16 @@ ExpandOptionName(const char * name, int depth)
 	 * If we are on the top level command line, then uppercase the
 	 * macro name for convenience.
 	 */
-	if ((depth == 0) && islower(*name))
+	if ((depth == 0) && isLower(*name))
 	{
-		strcpy(upname, name);
+		strcpy(upName, name);
 
-		upname[0] += 'A' - 'a';
+		upName[0] += 'A' - 'a';
 
-		name = upname;
+		name = upName;
 	}
 
-	if (islower(*name))
+	if (isLower(*name))
 	{
 		fprintf(stderr, "Option macro \"%s\" must be upper case\n",
 			name);
@@ -484,7 +531,7 @@ ExpandOptionName(const char * name, int depth)
 		return FALSE;
 	}
 
-	if (!ismacro(*name))
+	if (!isMacro(*name))
 	{
 		fprintf(stderr, "String value \"%s\" is not a macro name\n",
 			name);
@@ -506,11 +553,11 @@ ExpandOptionName(const char * name, int depth)
  * from the expansions, or -1 on an error.
  */
 int
-ExpandArguments(ARGS * ap, char ** table, int tablelen)
+ExpandArguments(ARGS * ap, char ** table, int tableLen)
 {
 	char *	name;
 	int	count;
-	int	subcount;
+	int	subCount;
 	ARGS	args;
 
 	count = 0;
@@ -523,7 +570,7 @@ ExpandArguments(ARGS * ap, char ** table, int tablelen)
 		/*
 		 * Make sure there is room left in the table.
 		 */
-		if (tablelen <= 0)
+		if (tableLen <= 0)
 		{
 			fprintf(stderr, "Too many arguments\n");
 
@@ -534,10 +581,10 @@ ExpandArguments(ARGS * ap, char ** table, int tablelen)
 		 * If the argument is not a macro name, then just put it
 		 * into the returned argument table 
 		 */
-		if (!ismacro(*name))
+		if (!isMacro(*name))
 		{
 			*table++ = name;
-			tablelen--;
+			tableLen--;
 			count++;
 
 			continue;
@@ -554,18 +601,18 @@ ExpandArguments(ARGS * ap, char ** table, int tablelen)
 		 * Add the result of the expansion into the table at this
 		 * point, possibly doing a recursive expansion.
 		 */
-		subcount = ExpandArguments(&args, table, tablelen);
+		subCount = ExpandArguments(&args, table, tableLen);
 
-		if (subcount < 0)
+		if (subCount < 0)
 			return -1;
 
 		/*
 		 * Increment past the number of entries that the expansion
 		 * added to the table.
 		 */
-		table += subcount;
-		tablelen -= subcount;
-		count += subcount;
+		table += subCount;
+		tableLen -= subCount;
+		count += subCount;
 	}
 
 	return count;
@@ -578,7 +625,7 @@ ExpandArguments(ARGS * ap, char ** table, int tablelen)
 static BOOL
 SetSeparation(ARGS * ap)
 {
-	char *	str;
+	const const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -706,35 +753,46 @@ DefaultOneOption(int id)
 			break;
 
 		case OPT_WIDTH:
-			width = DEFAULT_WIDTH;
+			outputWidth = DEFAULT_WIDTH;
 			break;
 
 		case OPT_SLEEP_TIME:
-			sleep_time_ms = DEFAULT_SLEEP_SEC * 1000;
+			sleepTimeMs = DEFAULT_SLEEP_SEC * 1000;
 			break;
 
 		case OPT_INFO:
-			info = FALSE;
+			isInfoShown = FALSE;
 			break;
 
 		case OPT_NO_SELF:
-			no_self = FALSE;
+			noSelf = FALSE;
 			break;
 
 		case OPT_MY_PROCS:
-			my_procs = FALSE;
+			myProcs = FALSE;
 			break;
 
+		case OPT_SHOW_THREADS:
+			showThreads = FALSE;
+			break;
+
+		case OPT_USE_THREADS:
+			useThreads = FALSE;
+			break;
+
+		case OPT_NO_COPY:
+			noCopy = FALSE;
+
 		case OPT_ACTIVE_ONLY:
-			active_only = FALSE;
+			activeOnly = FALSE;
 			break;
 
 		case OPT_NO_ROOT:
-			no_root = FALSE;
+			noRoot = FALSE;
 			break;
 
 		case OPT_CLEAR_SCREEN:
-			clear_screen = FALSE;
+			clearScreen = FALSE;
 			break;
 
 		case OPT_NORMAL_SORT:
@@ -746,15 +804,15 @@ DefaultOneOption(int id)
 			break;
 
 		case OPT_LOOP:
-			loop = FALSE;
+			isLooping = FALSE;
 			break;
 
 		case OPT_VERTICAL:
-			vertical = FALSE;
+			isVertical = FALSE;
 			break;
 
 		case OPT_INIT_TIME:
-			init_time = DEFAULT_INIT_SEC;
+			initSleepTime = DEFAULT_INIT_SEC;
 			break;
 
 		case OPT_CONDITION:
@@ -762,31 +820,31 @@ DefaultOneOption(int id)
 			break;
 
 		case OPT_TOP_COUNT:
-			top_count = 0;
+			topCount = 0;
 			break;
 
 		case OPT_NO_HEADER:
-			no_header = FALSE;
+			noHeader = FALSE;
 			break;
 
 		case OPT_PIDS:
-			pid_count = 0;
+			pidCount = 0;
 			break;
 
 		case OPT_USERS:
-			user_count = 0;
+			userCount = 0;
 			break;
 
 		case OPT_GROUPS:
-			group_count = 0;
+			groupCount = 0;
 			break;
 
 		case OPT_PROGRAMS:
-			program_count = 0;
+			programCount = 0;
 			break;
 
 		case OPT_ACTIVE_TIME:
-			active_time = DEFAULT_ACTIVE_SEC;
+			activeTime = DEFAULT_ACTIVE_SEC;
 			break;
 
 		case OPT_DEATH_TIME:
@@ -794,7 +852,7 @@ DefaultOneOption(int id)
 			break;
 
 		case OPT_SYNC_TIME:
-			sync_time = DEFAULT_SYNC_SEC;
+			syncTime = DEFAULT_SYNC_SEC;
 			break;
 
 		case OPT_COLUMN_WIDTH:
@@ -821,6 +879,10 @@ DefaultOneOption(int id)
 			ReplaceString(&displayName, NULL);
 			break;
 
+		case OPT_PERCENT_SECONDS:
+			percentSeconds = PERCENT_DEFAULT_SECONDS;
+			break;
+
 		case OPT_SCROLL_TIME:
 			scrollSeconds = DEFAULT_SCROLL_SEC;
 			break;
@@ -829,6 +891,18 @@ DefaultOneOption(int id)
 			overlapLines = DEFAULT_OVERLAP_LINES;
 			break;
 
+		case OPT_INFO_COLOR:
+			infoColorId = DEFAULT_COLOR_ID;
+			break;
+
+		case OPT_HEADER_COLOR:
+			headerColorId = DEFAULT_COLOR_ID;
+			break;
+
+		case OPT_ROW_COLOR:
+			ClearRowColorConditions();
+			break;
+
 		default:
 			break;
 	}
@@ -841,7 +915,7 @@ DefaultOneOption(int id)
 static BOOL
 SetCondition(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -864,7 +938,7 @@ SetCondition(ARGS * ap)
 static BOOL
 SetSleepTime(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -876,9 +950,9 @@ SetSleepTime(ARGS * ap)
 	ap->count--;
 	str = *ap->table++;
 
-	sleep_time_ms = (int) (GetFloatingNumber(&str) * 1000.0);
+	sleepTimeMs = (int) (GetFloatingNumber(&str) * 1000.0);
 
-	if (*str || (sleep_time_ms < 0))
+	if (*str || (sleepTimeMs < 0))
 	{
 		fprintf(stderr, "Bad sleep time argument\n");
 
@@ -895,7 +969,7 @@ SetSleepTime(ARGS * ap)
 static BOOL
 SetActiveTime(ARGS * ap)
 {
-	char *	str;
+	const const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -907,9 +981,9 @@ SetActiveTime(ARGS * ap)
 	ap->count--;
 	str = *ap->table++;
 
-	active_time = GetDecimalNumber(&str);
+	activeTime = GetDecimalNumber(&str);
 
-	if (*str || (active_time < 0))
+	if (*str || (activeTime < 0))
 	{
 		fprintf(stderr, "Bad active time argument\n");
 
@@ -926,7 +1000,7 @@ SetActiveTime(ARGS * ap)
 static BOOL
 SetDeathTime(ARGS * ap)
 {
-	char *	str;
+	const const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -957,7 +1031,7 @@ SetDeathTime(ARGS * ap)
 static BOOL
 SetSyncTime(ARGS * ap)
 {
-	char *	str;
+	const const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -969,9 +1043,9 @@ SetSyncTime(ARGS * ap)
 	ap->count--;
 	str = *ap->table++;
 
-	sync_time = GetDecimalNumber(&str);
+	syncTime = GetDecimalNumber(&str);
 
-	if (*str || (sync_time < 0))
+	if (*str || (syncTime < 0))
 	{
 		fprintf(stderr, "Bad sync time argument\n");
 
@@ -990,24 +1064,24 @@ SetSyncTime(ARGS * ap)
 static BOOL
 SetTopCount(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
-	top_set = TRUE;
+	isTopMode = TRUE;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
-		top_count = height - 1;
+		topCount = outputHeight - 1;
 
-		if (!no_header)
-			top_count--;
+		if (!noHeader)
+			topCount--;
 
-		if (info)
-			top_count--;
+		if (isInfoShown)
+			topCount--;
 
-		if (top_count <= 0)
-			top_count = 1;
+		if (topCount <= 0)
+			topCount = 1;
 
-		top_auto = TRUE;
+		isTopAuto = TRUE;
 
 		return TRUE;
 	}
@@ -1015,16 +1089,16 @@ SetTopCount(ARGS * ap)
 	ap->count--;
 	str = *ap->table++;
 
-	top_count = GetDecimalNumber(&str);
+	topCount = GetDecimalNumber(&str);
 
-	if (*str || (top_count < 0))
+	if (*str || (topCount < 0))
 	{
 		fprintf(stderr, "Bad top argument\n");
 
 		return FALSE;
 	}
 
-	top_auto = FALSE;
+	isTopAuto = FALSE;
 
 	return TRUE;
 }
@@ -1036,7 +1110,7 @@ SetTopCount(ARGS * ap)
 static BOOL
 SetPids(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 	pid_t	pid;
 	int	i;
 
@@ -1061,23 +1135,23 @@ SetPids(ARGS * ap)
 			return FALSE;
 		}
 
-		for (i = 0; i < pid_count; i++)
+		for (i = 0; i < pidCount; i++)
 		{
-			if (pid_list[i] == pid)
+			if (pidList[i] == pid)
 				break;
 		}
 
-		if (i < pid_count)
+		if (i < pidCount)
 			continue;
 
-		if (pid_count >= MAX_PIDS)
+		if (pidCount >= MAX_PIDS)
 		{
 			fprintf(stderr, "Too many pids specified\n");
 
 			return FALSE;
 		}
 
-		pid_list[pid_count++] = pid;
+		pidList[pidCount++] = pid;
 	}
 
 	return TRUE;
@@ -1090,7 +1164,7 @@ SetPids(ARGS * ap)
 static BOOL
 SetUsers(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 	uid_t	uid;
 	int	i;
 
@@ -1108,7 +1182,7 @@ SetUsers(ARGS * ap)
 		ap->count--;
 		str = *ap->table++;
 
-		if (isdigit(*str))
+		if (isDigit(*str))
 		{
 			uid = (uid_t) GetDecimalNumber(&str);
 
@@ -1131,23 +1205,23 @@ SetUsers(ARGS * ap)
 			}
 		}
 
-		for (i = 0; i < user_count; i++)
+		for (i = 0; i < userCount; i++)
 		{
-			if (user_list[i] == uid)
+			if (userList[i] == uid)
 				break;
 		}
 
-		if (i < user_count)
+		if (i < userCount)
 			continue;
 
-		if (user_count >= MAX_USERS)
+		if (userCount >= MAX_USERS)
 		{
 			fprintf(stderr, "Too many users specified\n");
 
 			return FALSE;
 		}
 
-		user_list[user_count++] = uid;
+		userList[userCount++] = uid;
 	}
 
 	return TRUE;
@@ -1160,7 +1234,7 @@ SetUsers(ARGS * ap)
 static BOOL
 SetGroups(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 	gid_t	gid;
 	int	i;
 
@@ -1178,7 +1252,7 @@ SetGroups(ARGS * ap)
 		ap->count--;
 		str = *ap->table++;
 
-		if (isdigit(*str))
+		if (isDigit(*str))
 		{
 			gid = (gid_t) GetDecimalNumber(&str);
 
@@ -1201,23 +1275,23 @@ SetGroups(ARGS * ap)
 			}
 		}
 
-		for (i = 0; i < group_count; i++)
+		for (i = 0; i < groupCount; i++)
 		{
-			if (group_list[i] == gid)
+			if (groupList[i] == gid)
 				break;
 		}
 
-		if (i < group_count)
+		if (i < groupCount)
 			continue;
 
-		if (group_count >= MAX_GROUPS)
+		if (groupCount >= MAX_GROUPS)
 		{
 			fprintf(stderr, "Too many groups specified\n");
 
 			return FALSE;
 		}
 
-		group_list[group_count++] = gid;
+		groupList[groupCount++] = gid;
 	}
 
 	return TRUE;
@@ -1252,23 +1326,23 @@ SetPrograms(ARGS * ap)
 		if (strlen(str) > MAX_PROGRAM_LEN)
 			continue;
 
-		for (i = 0; i < program_count; i++)
+		for (i = 0; i < programCount; i++)
 		{
-			if (strcmp(str, program_list[i]) == 0)
+			if (strcmp(str, programList[i]) == 0)
 				break;
 		}
 
-		if (i < program_count)
+		if (i < programCount)
 			continue;
 
-		if (program_count >= MAX_PROGRAMS)
+		if (programCount >= MAX_PROGRAMS)
 		{
 			fprintf(stderr, "Too many programss specified\n");
 
 			return FALSE;
 		}
 
-		strcpy(program_list[program_count++], str);
+		strcpy(programList[programCount++], str);
 	}
 
 	return TRUE;
@@ -1281,7 +1355,7 @@ SetPrograms(ARGS * ap)
 static BOOL
 SetInitTime(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -1293,9 +1367,9 @@ SetInitTime(ARGS * ap)
 	ap->count--;
 	str = *ap->table++;
 
-	init_time = GetDecimalNumber(&str);
+	initSleepTime = GetDecimalNumber(&str);
 
-	if (*str || (init_time < 0))
+	if (*str || (initSleepTime < 0))
 	{
 		fprintf(stderr, "Bad init time argument\n");
 
@@ -1312,7 +1386,7 @@ SetInitTime(ARGS * ap)
 static BOOL
 SetWidth(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -1324,17 +1398,17 @@ SetWidth(ARGS * ap)
 	ap->count--;
 	str = *ap->table++;
 
-	width = GetDecimalNumber(&str);
+	outputWidth = GetDecimalNumber(&str);
 
-	if (*str || (width <= 0))
+	if (*str || (outputWidth <= 0))
 	{
 		fprintf(stderr, "Bad width argument\n");
 
 		return FALSE;
 	}
 
-	if (width > MAX_WIDTH)
-		width = MAX_WIDTH;
+	if (outputWidth > MAX_WIDTH)
+		outputWidth = MAX_WIDTH;
 
 	return TRUE;
 }
@@ -1347,10 +1421,10 @@ SetWidth(ARGS * ap)
 static BOOL
 SetColumnWidth(ARGS * ap)
 {
-	COLUMN *column;
-	char *	name;
-	char *	str;
-	int	width;
+	COLUMN *	column;
+	const char *	name;
+	const char *	str;
+	int		width;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -1380,7 +1454,7 @@ SetColumnWidth(ARGS * ap)
 	 */
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
-		column->width = column->initwidth;
+		column->width = column->initWidth;
 
 		return TRUE;
 	}
@@ -1416,7 +1490,7 @@ SetColumnWidth(ARGS * ap)
 static BOOL
 SetColumns(ARGS * ap)
 {
-	show_count = 0;
+	showCount = 0;
 
 	return AddColumns(ap);
 }
@@ -1447,7 +1521,7 @@ AddColumns(ARGS * ap)
 	/*
 	 * Default the insert position to be after all existing columns.
 	 */
-	pos = show_count;
+	pos = showCount;
 
 	/*
 	 * Go through the column names one by one, inserting them into
@@ -1462,11 +1536,11 @@ AddColumns(ARGS * ap)
 		 * argument is the column number to insert columns at.
 		 * Subtract one to convert it to an array offset.
 		 */
-		if (isdigit(*name))
+		if (isDigit(*name))
 		{
 			pos = 0;
 
-			while (isdigit(*name))
+			while (isDigit(*name))
 				pos = pos * 10 + *name++ - '0';
 
 			if (*name || (pos < 0))
@@ -1496,17 +1570,17 @@ AddColumns(ARGS * ap)
 		/*
 		 * Remove the column if it was already in the column list.
 		 */
-		for (j = 0; j < show_count; j++)
+		for (j = 0; j < showCount; j++)
 		{
-			if (column == show_list[j])
+			if (column == showList[j])
 			{
-				while (j < show_count)
+				while (j < showCount)
 				{
-					show_list[j] = show_list[j + 1];
+					showList[j] = showList[j + 1];
 					j++;
 				}
 
-				show_count--;
+				showCount--;
 			}
 		}
 
@@ -1521,14 +1595,14 @@ AddColumns(ARGS * ap)
 		if (pos < 0)
 			pos = 0;
 
-		if (pos > show_count)
-			pos = show_count;
+		if (pos > showCount)
+			pos = showCount;
 
-		for (j = show_count; j > pos; j--)
-			show_list[j] = show_list[j - 1];
+		for (j = showCount; j > pos; j--)
+			showList[j] = showList[j - 1];
 
-		show_list[pos++] = column;
-		show_list[++show_count] = NULL;
+		showList[pos++] = column;
+		showList[++showCount] = NULL;
 	}
 
 	return TRUE;
@@ -1566,17 +1640,17 @@ RemoveColumns(ARGS * ap)
 			return FALSE;
 		}
 
-		for (j = 0; j < show_count; j++)
+		for (j = 0; j < showCount; j++)
 		{
-			if (column == show_list[j])
+			if (column == showList[j])
 			{
-				while (j < show_count)
+				while (j < showCount)
 				{
-					show_list[j] = show_list[j + 1];
+					showList[j] = showList[j + 1];
 					j++;
 				}
 
-				show_count--;
+				showCount--;
 			}
 		}
 	}
@@ -1705,13 +1779,44 @@ SetDisplay(ARGS * ap)
 }
 
 
+/*
+ * Set the seconds for cpu percentage calculations.
+ */
+static BOOL
+SetPercentSeconds(ARGS * ap)
+{
+	const char *	str;
+
+	if ((ap->count <= 0) || (**ap->table == '-'))
+	{
+		fprintf(stderr, "Missing cpu percentage argument\n");
+
+		return FALSE;
+	}
+
+	ap->count--;
+	str = *ap->table++;
+
+	percentSeconds = GetDecimalNumber(&str);
+
+	if (*str || (percentSeconds < 0) || (percentSeconds > PERCENT_MAX_SECONDS))
+	{
+		fprintf(stderr, "Bad cpu percentage argument\n");
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
 /*
  * Set the scolling time.
  */
 static BOOL
 SetScrollTime(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -1742,7 +1847,7 @@ SetScrollTime(ARGS * ap)
 static BOOL
 SetOverlapLines(ARGS * ap)
 {
-	char *	str;
+	const char *	str;
 
 	if ((ap->count <= 0) || (**ap->table == '-'))
 	{
@@ -1767,6 +1872,107 @@ SetOverlapLines(ARGS * ap)
 }
 
 
+/*
+ * Set the color of the information line.
+ */
+static BOOL
+SetInfoColor(ARGS * ap)
+{
+	const char *	color;
+	int		colorId;
+
+	if ((ap->count <= 0) || (**ap->table == '-'))
+	{
+		fprintf(stderr, "Missing information color argument\n");
+
+		return FALSE;
+	}
+
+	ap->count--;
+	color = *ap->table++;
+
+	colorId = AllocateColor(color);
+
+	if (colorId == BAD_COLOR_ID)
+	{
+		fprintf(stderr, "Cannot allocate information color\n");
+
+		return FALSE;
+	}
+
+	infoColorId = colorId;
+
+	return TRUE;
+}
+
+
+/*
+ * Set the color of the header line.
+ */
+static BOOL
+SetHeaderColor(ARGS * ap)
+{
+	const char *	color;
+	int		colorId;
+
+	if ((ap->count <= 0) || (**ap->table == '-'))
+	{
+		fprintf(stderr, "Missing header color argument\n");
+
+		return FALSE;
+	}
+
+	ap->count--;
+	color = *ap->table++;
+
+	colorId = AllocateColor(color);
+
+	if (colorId == BAD_COLOR_ID)
+	{
+		fprintf(stderr, "Cannot allocate header color\n");
+
+		return FALSE;
+	}
+
+	headerColorId = colorId;
+
+	return TRUE;
+}
+
+
+/*
+ * Set a condition for coloring or rows.
+ */
+static BOOL
+SetRowColorCondition(ARGS * ap)
+{
+	char *	color;
+	char *	condition;
+
+	if ((ap->count <= 0) || (**ap->table == '-'))
+	{
+		fprintf(stderr, "Missing color\n");
+
+		return FALSE;
+	}
+
+	ap->count--;
+	color = *ap->table++;
+
+	if ((ap->count <= 0) || (**ap->table == '-'))
+	{
+		fprintf(stderr, "Missing condition\n");
+
+		return FALSE;
+	}
+
+	ap->count--;
+	condition = *ap->table++;
+
+	return ParseRowColorCondition(color, condition);
+}
+
+
 /*
  * Usage text
  */
@@ -1801,6 +2007,9 @@ static	const char *	const usage_text[] =
 "  -font <name>             Set font name for X11 (default \"fixed\")",
 "  -foreground <color>      Set foreground color for X11 (default \"black\")",
 "  -background <color>      Set background color for X11 (default \"white\")",
+"  -headercolor <colors>    Set colors for header line",
+"  -infocolor <colors>      Set colors for info line",
+"  -rowcolor <colors> <cond>  Set colors for rows satisfying condition",
 "",
 "<condition-options> restrict which processes can be displayed:",
 "  -noself                  Do not show the ips process itself",
@@ -1815,11 +2024,14 @@ static	const char *	const usage_text[] =
 "  -deathtime <seconds>     Set number of seconds to show dead processes",
 "  -top [count]             Show number of processes from the top or what fits",
 "  -cond <expr>             Show processes for which the expression is nonzero",
+"  -showthreads             Show threads instead of just processes",
 "",
 "<other-options> are:",
 "  -sleep <seconds>         Set the sleep interval between loops",
 "  -initsleep <seconds>     Set the initial sleep time for active checks",
 "  -synctime <seconds>      Synchronize with some status at least this often",
+"  -usethreads              Use thread information for process status",
+"  -percentseconds <seconds>  Set seconds for cpu percentages (0 to 20)",
 "  -read <filename>         Read definitions from the specified filename",
 "  -noinit                  Don't read \"~/.ipsrc\" (must be first option)",
 "  -end                     Used to end argument lists if needed",
