@@ -2,7 +2,7 @@
  * Configurable ps-like program.
  * Useful utility routines.
  *
- * Copyright (c) 2008 David I. Bell
+ * Copyright (c) 2010 David I. Bell
  * Permission is granted to use, distribute, or modify this source,
  * provided that this copyright notice remains intact.
  */
@@ -61,26 +61,26 @@ typedef	struct
 /*
  * Table of user names.
  */
-static	int		username_count;
-static	int		username_avail;
-static	NAME *		username_table;
+static	int		userNameCount;
+static	int		userNameAvail;
+static	NAME *		userNameTable;
 
 
 /*
  * Table of group names.
  */
-static	int		groupname_count;
-static	int		groupname_avail;
-static	NAME *		groupname_table;
+static	int		groupNameCount;
+static	int		groupNameAvail;
+static	NAME *		groupNameTable;
 
 
 /*
  * Table of device names.
  */
-static	BOOL		devname_collected;
-static	int		devname_count;
-static	int		devname_avail;
-static	DEVICE *	devname_table;
+static	BOOL		devNameCollected;
+static	int		devNameCount;
+static	int		devNameAvail;
+static	DEVICE *	devNameTable;
 
 
 /*
@@ -103,14 +103,14 @@ struct	TEMPSTR
 /*
  * Amount of space left in the most recently allocated buffer.
  */
-static	int	tempstr_avail;
+static	int	tempStrAvail;
 
 
 /*
  * Linked list of string buffers.
  * Only the first one in the list has space for new allocation.
  */
-static	TEMPSTR *	tempstr_list;
+static	TEMPSTR *	tempStrList;
 
 
 /*
@@ -128,7 +128,7 @@ typedef	struct	SHARED_STR	SHARED_STR;
 struct	SHARED_STR
 {
 	long		magic;		/* magic number */
-	int		tableindex;	/* index into table of the entry */
+	int		tableIndex;	/* index into table of the entry */
 	int		count;		/* reference counter */
 	int		len;		/* length of string */
 	SHARED_STR *	next;		/* next string in list */
@@ -136,7 +136,7 @@ struct	SHARED_STR
 };
 
 
-static	SHARED_STR *	shared_table[SHARED_HASH_SIZE];
+static	SHARED_STR *	sharedTable[SHARED_HASH_SIZE];
 
 
 /*
@@ -161,49 +161,49 @@ static	void	DeviceNameRecursion(char *, int, int);
 char *
 AllocateSharedString(const char * str, int len)
 {
-	int		begch;
-	int		midch;
-	int		endch;
-	int		tableindex;
+	int		begCh;
+	int		midCh;
+	int		endCh;
+	int		tableIndex;
 	SHARED_STR *	entry;
 
 	if (len < 0)
 		return NULL;
 
 	if (len == 0)
-		return empty_string;
+		return emptyString;
 
 	if ((len == 1) && (*str == '/'))
-		return root_string;
+		return rootString;
 
 	/*
 	 * Get the first, middle, and last characters for comparison.
 	 */
-	begch = str[0];
-	midch = str[len / 2];
-	endch = str[len - 1];
+	begCh = str[0];
+	midCh = str[len / 2];
+	endCh = str[len - 1];
 
 	/*
 	 * Hash the characters along with the string length to select
 	 * an offset into the hash table to use for this string.
 	 */
-	tableindex = len + (begch << 16) + (midch << 20) + (endch << 24);
-	tableindex = ((unsigned int) tableindex) % SHARED_HASH_SIZE;
+	tableIndex = len + (begCh << 16) + (midCh << 20) + (endCh << 24);
+	tableIndex = ((unsigned int) tableIndex) % SHARED_HASH_SIZE;
 
 	/*
 	 * Search the selected string list in the table for the string.
 	 * Test the string length and the first, middle, and last characters.
 	 * If they all match then compare the whole string explicitly.
 	 */
-	for (entry = shared_table[tableindex]; entry; entry = entry->next)
+	for (entry = sharedTable[tableIndex]; entry; entry = entry->next)
 	{
-		if ((entry->len != len) || (entry->buf[0] != begch))
+		if ((entry->len != len) || (entry->buf[0] != begCh))
 			continue;
 
-		if (entry->buf[len / 2] != midch)
+		if (entry->buf[len / 2] != midCh)
 			continue;
 
-		if (entry->buf[len - 1] != endch)
+		if (entry->buf[len - 1] != endCh)
 			continue;
 
 		if (memcmp(entry->buf, str, len) != 0)
@@ -228,11 +228,11 @@ AllocateSharedString(const char * str, int len)
 	if (entry == NULL)
 		return NULL;
 
-	entry->next = shared_table[tableindex];
-	shared_table[tableindex] = entry;
+	entry->next = sharedTable[tableIndex];
+	sharedTable[tableIndex] = entry;
 
 	entry->magic = SHARED_MAGIC;
-	entry->tableindex = tableindex;
+	entry->tableIndex = tableIndex;
 	entry->count = 1;
 	entry->len = len;
 
@@ -254,9 +254,9 @@ FreeSharedString(char * str)
 {
 	SHARED_STR *	entry;
 	SHARED_STR *	prev;
-	int		tableindex;
+	int		tableIndex;
 
-	if ((str == NULL) || (str == empty_string) || (str == root_string))
+	if ((str == NULL) || (str == emptyString) || (str == rootString))
 		return;
 
 	/*
@@ -265,10 +265,10 @@ FreeSharedString(char * str)
 	 */
 	entry = (SHARED_STR *) (str - SHARED_BUF_OFFSET);
 
-	tableindex = entry->tableindex;
+	tableIndex = entry->tableIndex;
 
-	if ((entry->magic != SHARED_MAGIC) || (tableindex < 0) ||
-		(tableindex >= SHARED_HASH_SIZE) || (entry->len < 0))
+	if ((entry->magic != SHARED_MAGIC) || (tableIndex < 0) ||
+		(tableIndex >= SHARED_HASH_SIZE) || (entry->len < 0))
 	{
 		fprintf(stderr, "Freeing bad shared string header\n");
 
@@ -289,11 +289,11 @@ FreeSharedString(char * str)
 	 * See if it is the first string in the hash table.
 	 * If so, then removing it is easy.
 	 */
-	prev = shared_table[tableindex];
+	prev = sharedTable[tableIndex];
 
 	if (prev == entry)
 	{
-		shared_table[tableindex] = entry->next;
+		sharedTable[tableIndex] = entry->next;
 
 		free((char *) entry);
 
@@ -344,18 +344,18 @@ AllocTempString(int len)
 		exit(1);
 	}
 
-	if (len > tempstr_avail)
+	if (len > tempStrAvail)
 	{
 		head = (TEMPSTR *) AllocMemory(sizeof(TEMPSTR));
 
-		head->next = tempstr_list;
-		tempstr_list = head;
+		head->next = tempStrList;
+		tempStrList = head;
 
-		tempstr_avail = TEMPSTR_ALLOC_SIZE;
+		tempStrAvail = TEMPSTR_ALLOC_SIZE;
 	}
 
-	cp = &tempstr_list->buf[TEMPSTR_ALLOC_SIZE - tempstr_avail];
-	tempstr_avail -= len;
+	cp = &tempStrList->buf[TEMPSTR_ALLOC_SIZE - tempStrAvail];
+	tempStrAvail -= len;
 
 	return cp;
 }
@@ -390,15 +390,15 @@ FreeTempStrings(void)
 {
 	TEMPSTR *	head;
 
-	while (tempstr_list)
+	while (tempStrList)
 	{
-		head = tempstr_list;
-		tempstr_list = head->next;
+		head = tempStrList;
+		tempStrList = head->next;
 
 		free((char *) head);
 	}
 
-	tempstr_avail = 0;
+	tempStrAvail = 0;
 }
 
 
@@ -518,13 +518,13 @@ MakePrintable(char * cp, int len)
 BOOL
 PatternMatch(const char * text, const char * pattern)
 {
-	const char *	retrypat;
-	const char *	retrytxt;
+	const char *	retryPat;
+	const char *	retryTxt;
 	int		ch;
-	BOOL		found;
+	BOOL		isFound;
 
-	retrypat = NULL;
-	retrytxt = NULL;
+	retryPat = NULL;
+	retryTxt = NULL;
 
 	while (*text || *pattern)
 	{
@@ -533,12 +533,12 @@ PatternMatch(const char * text, const char * pattern)
 		switch (ch)
 		{
 			case '*':  
-				retrypat = pattern;
-				retrytxt = text;
+				retryPat = pattern;
+				retryTxt = text;
 				break;
 
 			case '[':  
-				found = FALSE;
+				isFound = FALSE;
 
 				while ((ch = *pattern++) != ']')
 				{
@@ -549,13 +549,13 @@ PatternMatch(const char * text, const char * pattern)
 						return FALSE;
 
 					if (*text == ch)
-						found = TRUE;
+						isFound = TRUE;
 				}
 
-				if (!found)
+				if (!isFound)
 				{
-					pattern = retrypat;
-					text = ++retrytxt;
+					pattern = retryPat;
+					text = ++retryTxt;
 				}
 
 				/*
@@ -589,8 +589,8 @@ PatternMatch(const char * text, const char * pattern)
 
 				if (*text)
 				{
-					pattern = retrypat;
-					text = ++retrytxt;
+					pattern = retryPat;
+					text = ++retryTxt;
 					break;
 				}
 
@@ -615,26 +615,26 @@ CollectUserNames(void)
 	const struct passwd *	pwd;
 	NAME *			name;
 
-	if (username_count > 0)
+	if (userNameCount > 0)
 		return;
 
 	while ((pwd = getpwent()) != NULL)
 	{
-		if (username_count >= username_avail)
+		if (userNameCount >= userNameAvail)
 		{
-			username_avail += USERNAME_ALLOC;
+			userNameAvail += USERNAME_ALLOC;
 
-			username_table = (NAME *) realloc(username_table,
-				(username_avail * sizeof(NAME)));
+			userNameTable = (NAME *) realloc(userNameTable,
+				(userNameAvail * sizeof(NAME)));
 
-			if (username_table == NULL)
+			if (userNameTable == NULL)
 			{
 				fprintf(stderr, "Cannot allocate memory\n");
 				exit(1);
 			}
 		}
 
-		name = &username_table[username_count++];
+		name = &userNameTable[userNameCount++];
 
 		name->id.uid = pwd->pw_uid;
 		strncpy(name->name, pwd->pw_name, MAX_NAME_LEN);
@@ -655,8 +655,8 @@ FindUserName(uid_t uid)
 	const NAME *	user;
 	int		count;
 
-	user = username_table;
-	count = username_count;
+	user = userNameTable;
+	count = userNameCount;
 
 	while (count-- > 0)
 	{
@@ -680,8 +680,8 @@ FindUserId(const char * name)
 	const NAME *	user;
 	int		count;
 
-	user = username_table;
-	count = username_count;
+	user = userNameTable;
+	count = userNameCount;
 
 	while (count-- > 0)
 	{
@@ -705,26 +705,26 @@ CollectGroupNames(void)
 	const struct group *	grp;
 	NAME *			name;
 
-	if (groupname_count > 0)
+	if (groupNameCount > 0)
 		return;
 
 	while ((grp = getgrent()) != NULL)
 	{
-		if (groupname_count >= groupname_avail)
+		if (groupNameCount >= groupNameAvail)
 		{
-			groupname_avail += GROUPNAME_ALLOC;
+			groupNameAvail += GROUPNAME_ALLOC;
 
-			groupname_table = (NAME *) realloc(groupname_table,
-				(groupname_avail * sizeof(NAME)));
+			groupNameTable = (NAME *) realloc(groupNameTable,
+				(groupNameAvail * sizeof(NAME)));
 
-			if (groupname_table == NULL)
+			if (groupNameTable == NULL)
 			{
 				fprintf(stderr, "Cannot allocate memory\n");
 				exit(1);
 			}
 		}
 
-		name = &groupname_table[groupname_count++];
+		name = &groupNameTable[groupNameCount++];
 
 		name->id.gid = grp->gr_gid;
 		strncpy(name->name, grp->gr_name, MAX_NAME_LEN);
@@ -745,8 +745,8 @@ FindGroupName(gid_t gid)
 	const NAME *	group;
 	int		count;
 
-	group = groupname_table;
-	count = groupname_count;
+	group = groupNameTable;
+	count = groupNameCount;
 
 	while (count-- > 0)
 	{
@@ -770,8 +770,8 @@ FindGroupId(const char * name)
 	const NAME *	group;
 	int		count;
 
-	group = groupname_table;
-	count = groupname_count;
+	group = groupNameTable;
+	count = groupNameCount;
 
 	while (count-- > 0)
 	{
@@ -794,14 +794,14 @@ CollectDeviceNames(void)
 {
 	char	pathBuffer[MAX_DEV_PATH_LEN + 2];
 
-	if (devname_collected)
+	if (devNameCollected)
 		return;
 
 	strcpy(pathBuffer, DEVICE_DIR);
 
 	DeviceNameRecursion(pathBuffer, strlen(pathBuffer), MAX_DEV_DEPTH);
 
-	devname_collected = TRUE;
+	devNameCollected = TRUE;
 }
 
 
@@ -885,22 +885,22 @@ DeviceNameRecursion(char * pathBuffer, int usedLength, int maxDepth)
 		 */
 		if (strcmp(pathBuffer, DEVICE_NULL) == 0)
 		{
-			null_dev = statBuf.st_dev;
-			null_inode = statBuf.st_ino;
+			nullDevice = statBuf.st_dev;
+			nullInode = statBuf.st_ino;
 		}
 
 		/*
 		 * We want to store the device name.
 		 * Reallocate the table if necessary.
 		 */
-		if (devname_count >= devname_avail)
+		if (devNameCount >= devNameAvail)
 		{
-			devname_avail += DEVNAME_ALLOC;
+			devNameAvail += DEVNAME_ALLOC;
 
-			devname_table = (DEVICE *) realloc(devname_table,
-				(devname_avail * sizeof(DEVICE)));
+			devNameTable = (DEVICE *) realloc(devNameTable,
+				(devNameAvail * sizeof(DEVICE)));
 
-			if (devname_table == NULL)
+			if (devNameTable == NULL)
 			{
 				fprintf(stderr, "Cannot allocate memory\n");
 				exit(1);
@@ -911,7 +911,7 @@ DeviceNameRecursion(char * pathBuffer, int usedLength, int maxDepth)
 		 * Store the device information, including its path
 		 * beyond the /dev prefix.
 		 */
-		device = &devname_table[devname_count++];
+		device = &devNameTable[devNameCount++];
 
 		device->id = statBuf.st_rdev;
 		device->dev = statBuf.st_dev;
@@ -940,8 +940,8 @@ FindDeviceName(dev_t devid)
 	if (devid <= 0)
 		return "-";
 
-	device = devname_table;
-	count = devname_count;
+	device = devNameTable;
+	count = devNameCount;
 
 	while (count-- > 0)
 	{
@@ -965,8 +965,8 @@ FindDeviceFromInode(dev_t dev, ino_t inode)
 	const DEVICE *	device;
 	int		count;
 
-	device = devname_table;
-	count = devname_count;
+	device = devNameTable;
+	count = devNameCount;
 
 	while (count-- > 0)
 	{
@@ -987,31 +987,31 @@ FindDeviceFromInode(dev_t dev, ino_t inode)
  * An invalid character stops the parse.
  */
 long
-GetDecimalNumber(char ** cpp)
+GetDecimalNumber(const char ** cpp)
 {
 	long	value;
-	BOOL	neg;
-	char *	cp;
+	BOOL	isNeg;
+	const char *	cp;
 
 	cp = *cpp;
 
-	neg = FALSE;
+	isNeg = FALSE;
 
-	while (isblank(*cp))
+	while (isBlank(*cp))
 		cp++;
 
 	if (*cp == '-')
 	{
-		neg = TRUE;
+		isNeg = TRUE;
 		cp++;
 	}
 
 	value = 0;
 
-	while (isdigit(*cp))
+	while (isDigit(*cp))
 		value = value * 10 + *cp++ - '0';
 
-	if (neg)
+	if (isNeg)
 		value = -value;
 
 	*cpp = cp;
@@ -1027,36 +1027,36 @@ GetDecimalNumber(char ** cpp)
  * An invalid character stops the parse.
  */
 double
-GetFloatingNumber(char ** cpp)
+GetFloatingNumber(const char ** cpp)
 {
 	double	value;
 	double	scale;
-	BOOL	neg;
-	char *	cp;
+	BOOL	isNeg;
+	const char *	cp;
 
 	cp = *cpp;
 
-	neg = FALSE;
+	isNeg = FALSE;
 
-	while (isblank(*cp))
+	while (isBlank(*cp))
 		cp++;
 
 	if (*cp == '-')
 	{
-		neg = TRUE;
+		isNeg = TRUE;
 		cp++;
 	}
 
 	value = 0.0;
 	scale = 1.0;
 
-	while (isdigit(*cp))
+	while (isDigit(*cp))
 		value = value * 10 + *cp++ - '0';
 
 	if (*cp == '.')
 		cp++;
 
-	while (isdigit(*cp))
+	while (isDigit(*cp))
 	{
 		value = value * 10.0 + *cp++ - '0';
 		scale *= 10.0;
@@ -1064,7 +1064,7 @@ GetFloatingNumber(char ** cpp)
 
 	value /= scale;
 
-	if (neg)
+	if (isNeg)
 		value = -value;
 
 	*cpp = cp;
@@ -1076,7 +1076,8 @@ GetFloatingNumber(char ** cpp)
 /*
  * Calculate the elapsed time in milliseconds between two timevals.
  * The new timeval can be NULL in which case the current time is used.
- * If the time appears to go backwards then a value of -1 is returned.
+ * If the time appears to go backwards or if the old time is zero
+ * then a value of 0 is returned.
  */
 long
 ElapsedMilliSeconds(const struct timeval * oldTime,
@@ -1086,6 +1087,12 @@ ElapsedMilliSeconds(const struct timeval * oldTime,
 	long		elapsedMicroSeconds;
 	struct	timeval	currentTime;
 
+	/*
+	 * Assume no elapsed time if the old time is zero.
+	 */
+	if (oldTime->tv_sec == 0)
+		return 0;
+
 	/*
 	 * If no new time is given then get it now.
 	 */
@@ -1105,7 +1112,7 @@ ElapsedMilliSeconds(const struct timeval * oldTime,
 	}
 
 	if (elapsedSeconds < 0)
-		return -1;
+		return 0;
 
  	return (elapsedSeconds * 1000) + (elapsedMicroSeconds / 1000);
 }
