@@ -4,7 +4,7 @@
  * The compilation of this display device is optional based on the
  * definition of the X11 symbol in the Makefile.
  *
- * Copyright (c) 2008 David I. Bell
+ * Copyright (c) 2010 David I. Bell
  * Permission is granted to use, distribute, or modify this source,
  * provided that this copyright notice remains intact.
  */
@@ -36,6 +36,21 @@
 #define	COLUMN_REALLOC_SLOP_SIZE	20
 
 
+/*
+ * Storage of a single cell of the text.
+ * This consists of a character and a color id.
+ */
+typedef	short	CELL;
+
+
+/*
+ * Macros to manipulate the cell components
+ */
+#define	GETCHAR(cell)		((cell) >> 8)
+#define	GETCOLOR(cell)		((cell) & 0xff)
+#define	MAKECELL(ch, color)	(((ch) << 8) | (color))
+
+
 /*
  * Storage for a line of text.
  * Note: This is a varying size structure.
@@ -49,14 +64,14 @@ struct LINE
 	int	clearCount;	/* columns needing clearing after used */
 	int	usedCount;	/* number of columns currently in use */
 	int	alloc;		/* number of columns allocated in data */
-	char	data[1];	/* text for line (MUST BE LAST) */
+	CELL	data[1];	/* text for line (MUST BE LAST) */
 };
 
 
 /*
  * Allocation size for a LINE to hold a certain number of columns.
  */
-#define	LINE_SIZE(columns)	(sizeof(LINE) + (columns))
+#define	LINE_SIZE(columns)	(sizeof(LINE) + ((columns) * sizeof(CELL)))
 
 
 /*
@@ -71,7 +86,10 @@ struct LINE
 
 
 static	BOOL	X11Open(DISPLAY *);
+static	BOOL	X11DefineColor(DISPLAY *, int, const char *, const char *, int);
+static	void	X11CreateWindow(DISPLAY *);
 static	void	X11Close(DISPLAY *);
+static	void	X11SetColor(DISPLAY *, int);
 static	void	X11Refresh(DISPLAY *);
 static	void	X11BeginPage(DISPLAY *);
 static	void	X11PutChar(DISPLAY *, int);
@@ -89,10 +107,10 @@ static	BOOL	X11DoesScroll(DISPLAY *);
 
 static DISPLAY	x11Display =
 {
-	X11Open, X11Close, X11Refresh, X11BeginPage, X11PutChar,
-	X11PutString, X11PutBuffer, X11EndPage, X11EventWait,
-	X11InputReady, X11ReadChar, X11RingBell,
-	X11GetRows, X11GetCols, X11DoesScroll
+	X11Open, X11DefineColor, X11CreateWindow, X11Close, X11SetColor,
+	X11Refresh, X11BeginPage, X11PutChar, X11PutString, X11PutBuffer,
+	X11EndPage, X11EventWait, X11InputReady, X11ReadChar,
+	X11RingBell, X11GetRows, X11GetCols, X11DoesScroll
 };
 
 
@@ -104,12 +122,13 @@ static	int		screen;		/* screen number */
 static	Window		rootWid;	/* top level window id */
 static	Window		mainWid;	/* main window id */
 static	GC		textGC;		/* graphics context for text */
+static	GC		clearGC;	/* graphics context for clearing areas */
 static	XFontStruct *	font;		/* font for text */
 static	Colormap	colorMap;	/* default color map */
 static	long		black;		/* black pixel value */
 static	long		white;		/* white pixel value */
-static	long		foreground;	/* foreground color */
-static	long		background;	/* background color */
+static	long		foreground;	/* default foreground color */
+static	long		background;	/* default background color */
 static	int		charWidth;	/* width of characters */
 static	int		charHeight;	/* height of characters */
 static	int		charAscent;	/* ascent of characters */
@@ -120,12 +139,25 @@ static	int		textCol;	/* column number for new text */
 static	int		rows;		/* current number of rows of text */
 static	int		cols;		/* current number of columns of text */
 static	int		inputCount;	/* characters stored in input buffer */
-static	BOOL		mapped;		/* main window is mapped */
-static	BOOL		redraw;		/* main window needs redrawing */
+static	BOOL		isMapped;	/* main window is mapped */
+static	BOOL		isRedrawNeeded;		/* main window needs redrawing */
 static	LINE *		lines[MAX_ROWS];	/* lines of text */
 static	char		inputBuffer[INPUT_BUFFER_SIZE];	/* input buffer */
 
 
+/*
+ * Color information.
+ * The color tables are indexed by the color id.
+ */
+static	int		currentColorId;
+static	long		currentForegroundPixel;
+static	long		currentBackgroundPixel;
+static	int		currentColorFlags;
+static	long		foregroundColorTable[MAX_COLORS];
+static	long		backgroundColorTable[MAX_COLORS];
+static	int		colorFlagsTable[MAX_COLORS];
+
+
 /*
  * Local procedures.
  */
@@ -134,8 +166,10 @@ static	void	DoKeyPress(XKeyEvent *);
 static	void	DoConfigure(XConfigureEvent *);
 static	void	StoreCharacter(int row, int col, int ch);
 static	void	StoreEndOfLine(int row, int col);
+static	void	SetColor(int colorId);
 static	void	UpdateWindow(void);
 static	void	UpdateRow(int row);
+static	void	DrawText(int textX, int textY, const char * buffer, int count);
 static	void	ClearWindow(void);
 static	LINE *	GetLine(int row);
 
@@ -152,22 +186,14 @@ GetX11Display(void)
 
 /*
  * Open the display device.
+ * This doesn't yet create the window.
  */
 static BOOL
 X11Open(DISPLAY * ipsDisplay)
 {
-	XSizeHints	sizeHints;
-	XWMHints	wmHints;
-	XEvent		event;
 	XColor		cellColor;
 	XColor		nameColor;
-	int		displayWidth;
-	int		displayHeight;
-	int		parsedFlags;
-	int		parsedX;
-	int		parsedY;
-	int		tryCols;
-	int		tryRows;
+	int		colorId;
 
 	/*
 	 * Open the display.
@@ -178,6 +204,7 @@ X11Open(DISPLAY * ipsDisplay)
 		return FALSE;
 
 	screen = DefaultScreen(display);
+	rootWid = RootWindow(display, screen);
 
 	/*
 	 * Try to load the font to be used.
@@ -230,6 +257,63 @@ X11Open(DISPLAY * ipsDisplay)
 		background = cellColor.pixel;
 	}
 
+	/*
+	 * Set up the default color table information.
+	 */
+	currentColorId = DEFAULT_COLOR_ID;
+	currentForegroundPixel = foreground;
+	currentBackgroundPixel = background;
+	currentColorFlags = COLOR_FLAG_NONE;
+
+	for (colorId = 0; colorId < MAX_COLORS; colorId++)
+	{
+		foregroundColorTable[colorId] = foreground;
+		backgroundColorTable[colorId] = background;
+		colorFlagsTable[colorId] = COLOR_FLAG_NONE;
+	}
+
+	/*
+	 * Set up the graphics contexts.
+	 */
+	textGC = XCreateGC(display, rootWid, 0, NULL);
+
+	XSetForeground(display, textGC, foreground);
+	XSetBackground(display, textGC, background);
+	XSetFont(display, textGC, font->fid);
+
+	clearGC = XCreateGC(display, rootWid, 0, NULL);
+
+	XSetForeground(display, clearGC, background);
+
+	/*
+	 * Synchronize so we know we are ok.
+	 */
+	XSync(display, False);
+
+	/*
+	 * Ok, return success.
+	 */
+	return TRUE;
+}
+
+
+/*
+ * Create the output window.
+ */
+static void
+X11CreateWindow(DISPLAY * ipsDisplay)
+{
+	XSizeHints	sizeHints;
+	XWMHints	wmHints;
+	XEvent		event;
+	int		displayWidth;
+	int		displayHeight;
+	int		parsedFlags;
+	int		parsedX;
+	int		parsedY;
+	unsigned int	tryCols;
+	unsigned int	tryRows;
+
 	/*
 	 * Parse the geometry string to get the preferred window size.
 	 * Then constrain the result to reasonable values.
@@ -293,11 +377,6 @@ X11Open(DISPLAY * ipsDisplay)
 		sizeHints.y = parsedY;
 	}
 
-	/*
-	 * Create and configure the window.
-	 */
-	rootWid = RootWindow(display, screen);
-
 	mainWid = XCreateSimpleWindow(display, rootWid,
 		0, 0, mainWidth, mainHeight, 0, 0, background);
 
@@ -312,32 +391,18 @@ X11Open(DISPLAY * ipsDisplay)
 
 	XSetWMHints(display, mainWid, &wmHints);
 
-	/*
-	 * Set up the graphics context.
-	 */
-	textGC = XCreateGC(display, mainWid, 0, NULL);
-
-	XSetForeground(display, textGC, foreground);
-	XSetBackground(display, textGC, background);
-	XSetFont(display, textGC, font->fid);
-
 	/*
 	 * Map the window and wait for it to appear.
 	 */
 	XMapWindow(display, mainWid);
 
-	mapped = FALSE;
+	isMapped = FALSE;
 
-	while (!mapped)
+	while (!isMapped)
 	{
 		XNextEvent(display, &event);
 		DoEvent(&event);
 	}
-
-	/*
-	 * Ok, return success.
-	 */
-	return TRUE;
 }
 
 
@@ -354,10 +419,86 @@ X11Close(DISPLAY * ipsDisplay)
 }
 
 
+/*
+ * Define a foreground and background color pair to be associated
+ * with the specified color id, along with flags which modify the
+ * look of the characters.  Empty names indicate that the default
+ * colors are to be used.  Returns TRUE if the definition was
+ * successful.
+ */
+static BOOL
+X11DefineColor(DISPLAY * ipsDisplay, int colorId,
+	const char * foregroundName, const char * backgroundName,
+	int colorFlags)
+{
+	long	foregroundPixel = foreground;
+	long	backgroundPixel = background;
+	XColor	cellColor;
+	XColor	nameColor;
+
+	if ((colorId < 0) || (colorId >= MAX_COLORS))
+		return FALSE;
+
+	/*
+	 * Validate that the flags are known.
+	 */
+	if (colorFlags & ~(COLOR_FLAG_UNDERLINE|COLOR_FLAG_BOLD))
+		return FALSE;
+
+	/*
+	 * Allocate the foreground and background colors if they
+	 * are not empty strings.
+	 */
+	if (*foregroundName)
+	{
+		if (!XAllocNamedColor(display, colorMap, foregroundName,
+			&cellColor, &nameColor))
+		{
+			return FALSE;
+		}
+
+		foregroundPixel = cellColor.pixel;
+	}
+
+	if (*backgroundName)
+	{
+		if (!XAllocNamedColor(display, colorMap, backgroundName,
+			&cellColor, &nameColor))
+		{
+			return FALSE;
+		}
+
+		backgroundPixel = cellColor.pixel;
+	}
+
+	/*
+	 * Store the colors in the table and return success.
+	 */
+	foregroundColorTable[colorId] = foregroundPixel;
+	backgroundColorTable[colorId] = backgroundPixel;
+	colorFlagsTable[colorId] = colorFlags;
+
+	return TRUE;
+}
+
+
+/*
+ * Set the color id to be used for new output in the window.
+ */
+static void
+X11SetColor(DISPLAY * ipsDisplay, int colorId)
+{
+	if ((colorId < 0) || (colorId > MAX_COLORS))
+		colorId = DEFAULT_COLOR_ID;
+
+	currentColorId = colorId;
+}
+
+
 static void
 X11Refresh(DISPLAY * ipsDisplay)
 {
-	redraw = TRUE;
+	isRedrawNeeded = TRUE;
 }
 
 
@@ -367,7 +508,7 @@ X11BeginPage(DISPLAY * ipsDisplay)
 	/*
 	 * Clear the window if we need to redraw it.
 	 */
-	if (redraw)
+	if (isRedrawNeeded)
 		ClearWindow();
 
 	/*
@@ -486,7 +627,7 @@ X11EventWait(DISPLAY * ipsDisplay, int milliSeconds)
 	 * If there is no delay then do nothing.
 	 */
 	if (milliSeconds <= 0)
-		return redraw;
+		return isRedrawNeeded;
 
 	/*
 	 * Wait for some more events to arrive or until the timeout expires.
@@ -513,7 +654,7 @@ X11EventWait(DISPLAY * ipsDisplay, int milliSeconds)
 	/*
 	 * Return TRUE if the window needs redrawing.
 	 */
-	return redraw;
+	return isRedrawNeeded;
 }
 
 
@@ -621,18 +762,18 @@ DoEvent(XEvent * event)
 	switch (event->type)
 	{
 		case Expose:
-			mapped = TRUE;
-			redraw = TRUE;
+			isMapped = TRUE;
+			isRedrawNeeded = TRUE;
 
 			break;
 
 		case MapNotify:
-			mapped = TRUE;
+			isMapped = TRUE;
 
 			break;
 
 		case UnmapNotify:
-			mapped = FALSE;
+			isMapped = FALSE;
 
 			break;
 
@@ -709,7 +850,7 @@ DoConfigure(XConfigureEvent * cp)
 	{
 		rows = newRows;
 		cols = newCols;
-		redraw = TRUE;
+		isRedrawNeeded = TRUE;
 	}
 }
 
@@ -723,6 +864,7 @@ static void
 StoreCharacter(int row, int col, int ch)
 {
 	LINE *	line;
+	CELL	cell;
 
 	/*
 	 * Make sure the position is within the window.
@@ -736,6 +878,11 @@ StoreCharacter(int row, int col, int ch)
 	 */
 	line = GetLine(row);
 
+	/*
+	 * Build the cell value from the character and the current color.
+	 */
+	cell = MAKECELL(ch, currentColorId);
+
 	/*
 	 * If the used part of the line is being extended than handle that.
 	 */
@@ -747,7 +894,7 @@ StoreCharacter(int row, int col, int ch)
 			line->beginUpdate = col;
 
 		line->endUpdate = col + 1;
-		line->data[col] = ch;
+		line->data[col] = cell;
 
 		return;
 	}
@@ -755,7 +902,7 @@ StoreCharacter(int row, int col, int ch)
 	/*
 	 * If a old column's character is being changed then handle that.
 	 */
-	if ((line->data[col] & 0xff) != (ch & 0xff))
+	if (line->data[col] != cell)
 	{
 		if (line->endUpdate == 0)
 			line->beginUpdate = col;
@@ -763,7 +910,7 @@ StoreCharacter(int row, int col, int ch)
 		if (col >= line->endUpdate)
 			line->endUpdate = col + 1;
 
-		line->data[col] = ch;
+		line->data[col] = cell;
 	}
 }
 
@@ -785,11 +932,17 @@ StoreEndOfLine(int row, int col)
 	if ((row < 0) || (row >= rows) || (col < 0) || (col >= cols))
 		return;
 
+	/*
+	 * Store end of lines in the rest of the row.
+	 */
+	while (col < cols)
+		StoreCharacter(row, col++, ' ');
+
 	/*
 	 * Get the LINE for this row which is large enough for the column.
 	 */
 	line = GetLine(row);
-
+#if 0
 	/*
 	 * If the unused column value is less then before then mark
 	 * the fact that the end of line needs clearing and set the
@@ -800,6 +953,7 @@ StoreEndOfLine(int row, int col)
 		line->clearCount = line->usedCount - col;
 		line->usedCount = col;
 	}
+#endif
 }
 
 
@@ -823,11 +977,15 @@ static void
 UpdateRow(int row)
 {
 	LINE *	line;
+	const CELL *	cell;
 	int	drawCount;
 	int	textX;
 	int	textY;
 	int	clearX;
 	int	clearY;
+	int	colorId;
+	int	runCount;
+	char	buffer[512];
 
 	line = GetLine(row);
 
@@ -836,25 +994,51 @@ UpdateRow(int row)
 	 */
 	drawCount = line->endUpdate - line->beginUpdate;
 
-	if (drawCount > 0)
+	/*
+	 * Calculate the window position of the change.
+	 */
+	textX = TEXT_OFF + charWidth * line->beginUpdate;
+	textY = TEXT_OFF + charAscent + charHeight * row;
+	cell = &line->data[line->beginUpdate];
+
+	/*
+	 * Loop drawing the characters until they are all done.
+	 */
+	while (drawCount > 0)
 	{
 		/*
-		 * Calculate the window position of the change.
+		 * Get the color of the first cell of the run
+		 * and make that color effective.
 		 */
-		textX = TEXT_OFF + charWidth * line->beginUpdate;
-		textY = TEXT_OFF + charAscent + charHeight * row;
+		colorId = GETCOLOR(*cell);
+
+		SetColor(colorId);
 
 		/*
-		 * Clear the area where the new text goes.
+		 * Count the number of characters in the run having
+		 * the same color and copy the characters themselves
+		 * into a local buffer while they fit.
 		 */
-		XClearArea(display, mainWid, textX, textY - charAscent,
-			charWidth * drawCount, charHeight, False);
+		runCount = 0;
+
+		while ((runCount < drawCount) && (runCount < sizeof(buffer)) &&
+			(GETCOLOR(*cell) == colorId))
+		{
+			buffer[runCount] = GETCHAR(*cell);
+			cell++;
+			runCount++;
+		}
 
 		/*
-		 * Draw the new text.
+		 * Draw the text at the specified location.
 		 */
-		XDrawString(display, mainWid, textGC, textX, textY,
-			line->data + line->beginUpdate, drawCount);
+		DrawText(textX, textY, buffer, runCount);
+
+		/*
+		 * Adjust the values for the characters used.
+		 */
+		textX += (charWidth * runCount);
+		drawCount -= runCount;
 	}
 
 	/*
@@ -878,6 +1062,92 @@ UpdateRow(int row)
 }
 
 
+/*
+ * Draw some text at the specified location.
+ */
+static void
+DrawText(int textX, int textY, const char * buffer, int count)
+{
+	/*
+	 * Clear or fill the area where the new text goes.
+	 */
+	if (currentBackgroundPixel == background)
+	{
+		XClearArea(display, mainWid, textX, textY - charAscent,
+			charWidth * count, charHeight, False);
+	}
+	else
+	{
+		XFillRectangle(display, mainWid, clearGC, textX, textY - charAscent,
+			charWidth * count, charHeight);
+	}
+
+	/*
+	 * Draw the new text.
+	 */
+	XDrawString(display, mainWid, textGC, textX, textY, buffer, count);
+
+	/*
+	 * If the text is bold then draw it again shifted by a pixel.
+	 * This is a bit of a hack but is easier than using multiple fonts.
+	 */
+	if (currentColorFlags & COLOR_FLAG_BOLD)
+	{
+		XDrawString(display, mainWid, textGC, textX + 1, textY, buffer, count);
+	}
+
+	/*
+	 * If the text is being underlined then do that.
+	 */
+	if (currentColorFlags & COLOR_FLAG_UNDERLINE)
+	{
+		XDrawLine(display, mainWid, textGC, textX, textY,
+			textX + charWidth * count - 1, textY);
+	}
+}
+
+
+/*
+ * Set the color being used for drawing text.
+ */
+void
+SetColor(int colorId)
+{
+	long foregroundPixel;
+	long backgroundPixel;
+
+	/*
+	 * Get the foreground and background pixel values
+	 * for the color id.
+	 */
+	if ((colorId < 0) || (colorId >= MAX_COLORS))
+		colorId = DEFAULT_COLOR_ID;
+
+	foregroundPixel = foregroundColorTable[colorId];
+	backgroundPixel = backgroundColorTable[colorId];
+
+	/*
+	 * Set the colors in the graphics contexts if necessary
+	 * and save the new values.
+	 */
+	if (foregroundPixel != currentForegroundPixel)
+		XSetForeground(display, textGC, foregroundPixel);
+
+	if (backgroundPixel != currentBackgroundPixel)
+	{
+		XSetBackground(display, textGC, backgroundPixel);
+		XSetForeground(display, clearGC, backgroundPixel);
+	}
+
+	/*
+	 * Save the values for checking elsewhere.
+	 */
+	currentForegroundPixel = foregroundPixel;
+	currentBackgroundPixel = backgroundPixel;
+	currentColorFlags = colorFlagsTable[colorId];
+}
+
+
 /*
  * Clear the window and reinitialise the LINE structures to be empty.
  * This is called after exposure or configuration events when we are
@@ -910,7 +1180,7 @@ ClearWindow(void)
 	/*
 	 * We now don't need to redraw anymore.
 	 */
-	redraw = FALSE;
+	isRedrawNeeded = FALSE;
 }
 
 
